
import math
import unittest

import paramak
import pytest


class TestPoloidalFieldCoilCaseSetFC(unittest.TestCase):

    def setUp(self):
        self.pf_coils_set = paramak.PoloidalFieldCoilSet(
            heights=[10, 10, 20, 20],
            widths=[10, 10, 20, 40],
            center_points=[(100, 100), (100, 150), (50, 200), (50, 50)],
        )

        self.test_shape = paramak.PoloidalFieldCoilCaseSetFC(
            pf_coils=self.pf_coils_set,
            casing_thicknesses=[5, 10, 5, 10],
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a PoloidalFieldCoilCaseSetFC are correct."""

        assert self.test_shape.rotation_angle == 360

    def test_processed_points_calculation(self):
        """Checks that the processed_points used to construct the
        PoloidalFieldCoilCaseSetFC are calculated correctly from the parameters given."""

        assert self.test_shape.processed_points == [
            (105.0, 105.0, 'straight'),
            (105.0, 95.0, 'straight'),
            (95.0, 95.0, 'straight'),
            (95.0, 105.0, 'straight'),
            (105.0, 105.0, 'straight'),
            (110.0, 110.0, 'straight'),
            (110.0, 90.0, 'straight'),
            (90.0, 90.0, 'straight'),
            (90.0, 110.0, 'straight'),
            (110.0, 110.0, 'straight'),
            (105.0, 155.0, 'straight'),
            (105.0, 145.0, 'straight'),
            (95.0, 145.0, 'straight'),
            (95.0, 155.0, 'straight'),
            (105.0, 155.0, 'straight'),
            (115.0, 165.0, 'straight'),
            (115.0, 135.0, 'straight'),
            (85.0, 135.0, 'straight'),
            (85.0, 165.0, 'straight'),
            (115.0, 165.0, 'straight'),
            (60.0, 210.0, 'straight'),
            (60.0, 190.0, 'straight'),
            (40.0, 190.0, 'straight'),
            (40.0, 210.0, 'straight'),
            (60.0, 210.0, 'straight'),
            (65.0, 215.0, 'straight'),
            (65.0, 185.0, 'straight'),
            (35.0, 185.0, 'straight'),
            (35.0, 215.0, 'straight'),
            (65.0, 215.0, 'straight'),
            (70.0, 60.0, 'straight'),
            (70.0, 40.0, 'straight'),
            (30.0, 40.0, 'straight'),
            (30.0, 60.0, 'straight'),
            (70.0, 60.0, 'straight'),
            (80.0, 70.0, 'straight'),
            (80.0, 30.0, 'straight'),
            (20.0, 30.0, 'straight'),
            (20.0, 70.0, 'straight'),
            (80.0, 70.0, 'straight'),
            (105.0, 105.0, 'straight')
        ]

    def test_points_calculation(self):
        """Checks that the points used to construct the PoloidalFieldCoilCaseSetFC are
        calculated correctly from the parameters given."""

        assert self.test_shape.points == [
            (105.0, 105.0),
            (105.0, 95.0),
            (95.0, 95.0),
            (95.0, 105.0),
            (105.0, 105.0),
            (110.0, 110.0),
            (110.0, 90.0),
            (90.0, 90.0),
            (90.0, 110.0),
            (110.0, 110.0),
            (105.0, 155.0),
            (105.0, 145.0),
            (95.0, 145.0),
            (95.0, 155.0),
            (105.0, 155.0),
            (115.0, 165.0),
            (115.0, 135.0),
            (85.0, 135.0),
            (85.0, 165.0),
            (115.0, 165.0),
            (60.0, 210.0),
            (60.0, 190.0),
            (40.0, 190.0),
            (40.0, 210.0),
            (60.0, 210.0),
            (65.0, 215.0),
            (65.0, 185.0),
            (35.0, 185.0),
            (35.0, 215.0),
            (65.0, 215.0),
            (70.0, 60.0),
            (70.0, 40.0),
            (30.0, 40.0),
            (30.0, 60.0),
            (70.0, 60.0),
            (80.0, 70.0),
            (80.0, 30.0),
            (20.0, 30.0),
            (20.0, 70.0),
            (80.0, 70.0),
        ]

    def test_from_pf_coil_set(self):
        """Checks that a set of PF coil cases can be constructed from a PF coils object
        using the PoloidalField~CoilCaseSetFC parametric shape."""

        assert self.test_shape.solid is not None
        assert len(self.test_shape.solid.Solids()) == 4
        assert len(self.pf_coils_set.solid.Solids()) == 4

    def test_with_zero_thickness(self):
        """Creates a set of PF coil cases from a PF coils object and sets one
        of the casing thicknesses to 0."""

        self.test_shape.casing_thicknesses = [5, 5, 0, 10]

        assert self.test_shape.solid is not None
        assert len(self.test_shape.solid.Solids()) == 3
        assert len(self.pf_coils_set.solid.Solids()) == 4

    def test_from_pf_coil_set_absolute_volume(self):
        """Creates a set of pf coil cases from a pf coil set object and checks
        that the volume is correct."""

        assert self.test_shape.volume() == pytest.approx((((20 * 5 * 2) + (10 * 5 * 2)) * math.pi * 2 * 100) + (((30 * 10 * 2) + (
            10 * 10 * 2)) * math.pi * 2 * 100) + (((30 * 5 * 2) + (20 * 5 * 2)) * math.pi * 2 * 50) + (((60 * 10 * 2) + (20 * 10 * 2)) * math.pi * 2 * 50))

    def test_from_pf_coil_set_absolute_areas(self):
        """Creates a set of pf coil cases from a pf coil set object and checks
        that the areas are correct"""

        assert len(self.test_shape.areas) == 32
        assert len(set([round(i) for i in self.test_shape.areas])) == 16
        assert self.test_shape.areas.count(
            pytest.approx(10 * math.pi * 2 * 100)) == 6
        assert self.test_shape.areas.count(
            pytest.approx(40 * math.pi * 2 * 50)) == 4
        assert self.test_shape.areas.count(
            pytest.approx(30 * math.pi * 2 * 100)) == 4
        assert self.test_shape.areas.count(
            pytest.approx(30 * math.pi * 2 * 50)) == 2
        assert self.test_shape.areas.count(
            pytest.approx(10 * math.pi * 2 * 105)) == 3
        assert self.test_shape.areas.count(
            pytest.approx(10 * math.pi * 2 * 95)) == 2
        assert self.test_shape.areas.count(
            pytest.approx(20 * math.pi * 2 * 110)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(20 * math.pi * 2 * 90)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(30 * math.pi * 2 * 115)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(30 * math.pi * 2 * 85)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(20 * math.pi * 2 * 60)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(20 * math.pi * 2 * 40)) == 2
        assert self.test_shape.areas.count(
            pytest.approx(30 * math.pi * 2 * 65)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(20 * math.pi * 2 * 70)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(20 * math.pi * 2 * 30)) == 1
        assert self.test_shape.areas.count(
            pytest.approx(40 * math.pi * 2 * 80)) == 1

    def test_PoloidalFieldCoilCaseSetFC_incorrect_lengths_FC(self):
        """Checks that an error is raised when a PoloidalFieldCoilCaseSetFC is made
        with the wrong number of casing thicknesses using a coil set object."""

        def make_PoloidalFieldCoilCaseSetFC_incorrect_lengths_FC():
            self.test_shape.casing_thicknesses = [5, 5, 10]
            self.test_shape.solid

        self.assertRaises(
            ValueError,
            make_PoloidalFieldCoilCaseSetFC_incorrect_lengths_FC
        )

    def test_PoloidalFieldCoilCaseSetFC_incorrect_lengths(self):
        """Checks that an error is raised when a PoloidalFieldCoilCaseSetFC is made
        with the wrong number of casing thicknesses using a list."""

        def make_PoloidalFieldCoilCaseSetFC_incorrect_lengths():
            self.pf_coils_set.height = 10
            self.pf_coils_set.width = 10
            self.pf_coils_set.center_point = (100, 100)

            self.test_shape.pf_coils = [self.pf_coils_set]
            self.test_shape.solid

        self.assertRaises(
            ValueError,
            make_PoloidalFieldCoilCaseSetFC_incorrect_lengths
        )

    def test_PoloidalFieldCoilCaseSetFC_incorrect_pf_coil(self):
        """Checks that an error is raised when a PoloidalFieldCoilCaseSetFC is made
        with the pf_coils as an incorrect entry."""

        def make_PoloidalFieldCoilCaseSetFC_incorrect_pf_coil():
            self.test_shape.pf_coils = 20
            self.test_shape.solid

        self.assertRaises(
            ValueError,
            make_PoloidalFieldCoilCaseSetFC_incorrect_pf_coil
        )

    def test_from_list(self):
        """Creates a set of PF coil cases from a list of PF coils with a list
        of thicknesses."""

        pf_coils_1 = paramak.PoloidalFieldCoil(height=10,
                                               width=10,
                                               center_point=(100, 100))

        pf_coils_2 = paramak.PoloidalFieldCoil(height=10,
                                               width=10,
                                               center_point=(100, 150))

        pf_coils_3 = paramak.PoloidalFieldCoil(height=20,
                                               width=20,
                                               center_point=(50, 200))

        pf_coils_4 = paramak.PoloidalFieldCoil(height=20,
                                               width=40,
                                               center_point=(50, 50))

        test_shape = paramak.PoloidalFieldCoilCaseSetFC(
            pf_coils=[pf_coils_1, pf_coils_2, pf_coils_3, pf_coils_4],
            casing_thicknesses=[5, 5, 10, 10]
        )

        assert test_shape.solid is not None
        assert len(test_shape.solid.Solids()) == 4

    def test_PoloidalFieldCoilCaseSetFC_with_number_thickness(self):
        """Creates a set of PF coil cases from a list of PF coils with a
        single numerical thicknesses."""

        pf_coils_1 = paramak.PoloidalFieldCoil(height=10,
                                               width=10,
                                               center_point=(100, 100))

        pf_coils_2 = paramak.PoloidalFieldCoil(height=10,
                                               width=10,
                                               center_point=(100, 150))

        pf_coils_3 = paramak.PoloidalFieldCoil(height=20,
                                               width=20,
                                               center_point=(50, 200))

        pf_coils_4 = paramak.PoloidalFieldCoil(height=20,
                                               width=40,
                                               center_point=(50, 50))

        test_shape = paramak.PoloidalFieldCoilCaseSetFC(
            pf_coils=[pf_coils_1, pf_coils_2, pf_coils_3, pf_coils_4],
            casing_thicknesses=10,
        )

        assert test_shape.casing_thicknesses == 10
        assert test_shape.solid is not None
        assert len(test_shape.solid.Solids()) == 4




import warnings
from typing import List, Optional, Tuple, Union, Callable

import numpy as np
from paramak import BlanketFP, RotateStraightShape
from paramak.utils import (cut_solid, distance_between_two_points, extend,
                           rotate)
from scipy.optimize import minimize


class BlanketFPPoloidalSegments(BlanketFP):
    """Poloidally segmented Blanket inheriting from paramak.BlanketFP.

    Args:
        segments_angles: If not None, the segments ends will
            be located at these angles. If None and if the constraints
            length_limits and nb_segments_limits are not None, segments angles
            will be linearly distributed. Else, an optimum configuration
            meeting the set requirements will be found. Defaults to None.
        num_segments: Number of segments (ignored if segments_angles is not
            None).
        length_limits: The minimum and maximum acceptable length of the
            segments. Ex: (100, 500), (100, None), (None, 300), None, (None,
            None).
        nb_segments_limits: The minimum and maximum acceptable number of
            segments. Ex: (3, 10), (5, None), (None, 7), None, (None, None).
        segments_gap: Distance between segments.
    """

    def __init__(
        self,
        segments_angles: Optional[List[float]] = None,
        num_segments: Optional[int] = 7,
        length_limits: Optional[Tuple[Union[float, None], Union[float, None]]] = None,
        nb_segments_limits: Optional[Tuple[float, float]] = None,
        segments_gap: Optional[float] = 0.0,
        **kwargs
    ):
        super().__init__(**kwargs)

        self.num_segments = num_segments
        self.length_limits = length_limits
        self.nb_segments_limits = nb_segments_limits
        self.segments_angles = segments_angles
        self.segments_gap = segments_gap
        self.segments_cutters = None

    @property
    def segments_angles(self):
        return self._segments_angles

    @segments_angles.setter
    def segments_angles(self, value):
        if value is not None:
            if self.start_angle is not None or self.stop_angle is not None:
                msg = "start_angle and stop_angle attributes will be " + \
                    "ignored if segments_angles is not None"
                warnings.warn(msg)
            elif self.num_segments is not None:
                msg = "num_segment attribute will be ignored if " + \
                    "segments_angles is not None"
                warnings.warn(msg)
        self._segments_angles = value

    @property
    def num_segments(self):
        return self._num_segments

    @num_segments.setter
    def num_segments(self, value):
        if value is not None:
            self.num_points = value + 1
        self._num_segments = value

    @property
    def segments_cutters(self):
        self.create_segment_cutters()
        return self._segments_cutters

    @segments_cutters.setter
    def segments_cutters(self, value):
        self._segments_cutters = value

    def get_angles(self):
        """Get the poloidal angles of the segments.

        Returns:
            list: the angles
        """
        if (self.length_limits, self.nb_segments_limits) != (None, None):
            angles = segments_optimiser(
                self.length_limits, self.nb_segments_limits,
                self.distribution, (self.start_angle, self.stop_angle),
                stop_on_success=True
            )
        elif self.segments_angles is None:
            angles = np.linspace(
                self.start_angle, self.stop_angle,
                num=self.num_segments + 1)
        else:
            angles = self.segments_angles
        return angles

    def find_points(self):
        points = super().find_points(angles=self.get_angles())

        # every points straight connections
        for point in points:
            point[-1] = 'straight'
        self.points = points[:-1]

    def create_solid(self):
        solid = super().create_solid()
        segments_cutters = self.segments_cutters
        if segments_cutters is not None:
            solid = cut_solid(solid, segments_cutters)
        self.solid = solid
        return solid

    def create_segment_cutters(self):
        """Creates a shape for cutting the blanket into segments and store it
        in segments_cutter attribute
        """
        if self.segments_gap > 0:
            # initialise main cutting shape
            cutting_shape = RotateStraightShape(
                rotation_angle=self.rotation_angle,
                azimuth_placement_angle=self.azimuth_placement_angle,
                union=[])
            # add points to the shape to avoid void solid
            cutting_shape.points = [
                (self.major_radius,
                 self.vertical_displacement),
                (self.major_radius +
                 self.minor_radius /
                 10,
                 self.vertical_displacement),
                (self.major_radius +
                 self.minor_radius /
                 10,
                 self.vertical_displacement +
                 self.minor_radius /
                 10),
                (self.major_radius,
                 self.vertical_displacement +
                 self.minor_radius /
                 10),
            ]

            # Create cutters for each gap
            for inner_point, outer_point in zip(
                    self.inner_points[:-1],
                    self.outer_points[-1::-1]):
                # initialise cutter for gap
                cutter = RotateStraightShape(
                    rotation_angle=self.rotation_angle,
                    azimuth_placement_angle=self.azimuth_placement_angle
                )
                # create rectangle of dimension |AB|*2.6 x self.segments_gap
                A = (inner_point[0], inner_point[1])
                B = (outer_point[0], outer_point[1])

                # increase rectangle length
                security_factor = 0.8
                local_thickness = distance_between_two_points(A, B)
                A = extend(A, B, -local_thickness * security_factor)
                B = extend(A, B, local_thickness * (1 + 2 * security_factor))
                # create points for cutter
                points_cutter = [
                    A,
                    B,
                    rotate(
                        B,
                        extend(
                            B,
                            A,
                            self.segments_gap),
                        angle=-np.pi / 2),
                    rotate(
                        A,
                        extend(
                            A,
                            B,
                            self.segments_gap),
                        angle=np.pi / 2)]
                cutter.points = points_cutter
                # add cutter to global cutting shape
                cutting_shape.union.append(cutter)

            self.segments_cutters = cutting_shape


def compute_lengths_from_angles(
        angles: List[float],
        distribution: Callable):
    """Computes the length of segments between a set of points on a (x,y)
    distribution.

    Args:
        angles (list): Contains the angles of the points (degree)
        distribution (callable): function taking an angle as argument and
            returning (x,y) coordinates.

    Returns:
        list: contains the lengths of the segments.
    """
    points = []
    for angle in angles:
        points.append(distribution(angle))

    lengths = []
    for i in range(len(points) - 1):
        lengths.append(distance_between_two_points(points[i], points[i + 1]))
    return lengths


def segments_optimiser(length_limits, nb_segments_limits, distribution, angles,
                       stop_on_success=True):
    """Optimiser segmenting a given R(theta), Z(theta) distribution of points
    with constraints regarding the number of segments and the length of the
    segments.

    Args:
        length_limits ((float, float)): The minimum and maximum acceptable
            length of the segments. Ex: (100, 500), (100, None), (None, 300),
            None, (None, None)
        nb_segments_limits ((int, int)): The minimum and maximum acceptable
            number of segments. Ex: (3, 10), (5, None), (None, 7),
            None, (None, None)
        distribution (callable): function taking an angle as argument and
            returning (x,y) coordinates.
        angles ((float, float)): the start and stop angles of the distribution.
        stop_on_sucess (bool, optional): If set to True, the optimiser will
            stop as soon as a configuration meets the requirements.

    Returns:
        list: list of optimised angles
    """
    if length_limits is None:
        min_length, max_length = None, None
    else:
        min_length, max_length = length_limits

    if nb_segments_limits is None:
        min_nb_segments, max_nb_segments = None, None
    else:
        min_nb_segments, max_nb_segments = nb_segments_limits

    if min_length is None:
        min_length = 0
    if max_length is None:
        max_length = float('inf')
    if min_nb_segments is None:
        min_nb_segments = 1
    if max_nb_segments is None:
        max_nb_segments = 50

    start_angle, stop_angle = angles

    # define cost function
    def cost_function(angles):
        angles_with_extremums = [start_angle, *angles, stop_angle]

        lengths = compute_lengths_from_angles(
            angles_with_extremums, distribution)

        cost = 0
        for length in lengths:
            if not min_length <= length <= max_length:
                cost += min(abs(min_length - length), abs(max_length - length))
        return cost

    # test for several numbers of segments the best config
    best = [float("inf"), []]

    for nb_segments in range(min_nb_segments, max_nb_segments + 1):
        # initialise angles to linspace
        list_of_angles = \
            np.linspace(start_angle, stop_angle, num=nb_segments + 1)

        # use scipy minimize to find best set of angles
        res = minimize(
            cost_function, list_of_angles[1:-1], method="Nelder-Mead")

        # complete the optimised angles with extrema
        optimised_angles = [start_angle] + \
            [angle for angle in res.x] + [stop_angle]

        # check that the optimised angles meet the lengths requirements
        lengths = compute_lengths_from_angles(optimised_angles, distribution)
        break_the_rules = False
        for length in lengths:
            if not min_length <= length <= max_length:
                break_the_rules = True
                break
        if not break_the_rules:
            # compare with previous results and get the minimum
            # cost function value
            best = min(best, [res.fun, optimised_angles], key=lambda x: x[0])
            if stop_on_success:
                return optimised_angles

    # return the results
    returned_angles = best[1]
    if returned_angles == []:
        msg = "Couldn't find optimum configuration for Blanket segments"
        raise ValueError(msg)

    return returned_angles




import unittest

import paramak
import pytest


class TestToroidalFieldCoilPrincetonD(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.ToroidalFieldCoilPrincetonD(
            R1=100, R2=300, thickness=50, distance=30, number_of_coils=1,
            with_inner_leg=True
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a ToroidalFieldCoilPrincetonD are correct."""

        assert self.test_shape.rotation_angle == 360
        assert self.test_shape.vertical_displacement == 0
        assert self.test_shape.with_inner_leg

    def test_creation_with_inner_leg(self):
        """Creates a tf coil with inner leg using the ToroidalFieldCoilPrincetonD
        parametric component and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000
        assert self.test_shape.inner_leg_connection_points is not None

        test_inner_leg = paramak.ExtrudeStraightShape(
            points=self.test_shape.inner_leg_connection_points, distance=30
        )
        assert test_inner_leg.solid is not None

    def test_creation_no_inner_leg(self):
        """Creates a tf coil with no inner leg using the ToroidalFieldCoilPrincetonD
        parametric component and checks that a cadquery solid is created."""

        test_volume = self.test_shape.volume()

        test_inner_leg = paramak.ExtrudeStraightShape(
            points=self.test_shape.inner_leg_connection_points, distance=30
        )
        inner_leg_volume = test_inner_leg.volume()

        self.test_shape.with_inner_leg = False

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() < test_volume

    def test_relative_volume(self):
        """Creates tf coil shapes with different numbers of tf coils and checks that
        their relative volumes are correct."""

        self.test_shape.number_of_coils = 4
        test_volume = self.test_shape.volume()

        self.test_shape.number_of_coils = 8
        assert test_volume == pytest.approx(
            self.test_shape.volume() * 0.5, rel=0.01)

    def test_rotation_angle(self):
        """Creates a tf coil with a rotation_angle < 360 degrees and checks
        that the correct cut is performed and the volume is correct."""

        self.test_shape.number_of_coils = 8

        self.test_shape.rotation_angle = 360
        self.test_shape.workplane = "XZ"
        test_volume = self.test_shape.volume()
        self.test_shape.rotation_angle = 180
        assert self.test_shape.volume() == pytest.approx(
            test_volume * 0.5, rel=0.01)

        self.test_shape.rotation_angle = 360
        self.test_shape.workplane = "YZ"
        test_volume = self.test_shape.volume()
        self.test_shape.rotation_angle = 180
        assert self.test_shape.volume() == pytest.approx(
            test_volume * 0.5, rel=0.01)

        self.test_shape.rotation_angle = 360
        self.test_shape.workplane = "XY"
        self.test_shape.rotation_axis = "Y"
        test_volume = self.test_shape.volume()
        self.test_shape.rotation_angle = 180
        assert self.test_shape.volume() == pytest.approx(
            test_volume * 0.5, rel=0.01)



import math
import os
import unittest
from pathlib import Path

import pytest
from paramak import ExtrudeCircleShape


class TestExtrudeCircleShape(unittest.TestCase):

    def setUp(self):
        self.test_shape = ExtrudeCircleShape(
            points=[(30, 0)], radius=10, distance=30
        )

    def test_default_parameters(self):
        """Checks that the default parameters of an ExtrudeCircleShape are correct."""

        assert self.test_shape.rotation_angle == 360
        assert self.test_shape.extrude_both

    def test_absolute_shape_volume(self):
        """Creates an ExtrudeCircleShape and checks that its volume is correct."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() == pytest.approx(math.pi * (10**2) * 30)

    def test_relative_shape_volume(self):
        """Creates two ExtrudeCircleShapes and checks that their relative
        volumes are correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.azimuth_placement_angle = [0, 90, 180, 270]
        assert test_volume * 4 == pytest.approx(self.test_shape.volume())

    def test_absolute_shape_areas(self):
        """Creates ExtrudeCircleShapes and checks that the areas of each face
        are correct."""

        assert self.test_shape.area == pytest.approx(
            (math.pi * (10**2) * 2) + (math.pi * (2 * 10) * 30)
        )
        assert len(self.test_shape.areas) == 3
        assert self.test_shape.areas.count(
            pytest.approx(math.pi * (10**2))) == 2
        assert self.test_shape.areas.count(
            pytest.approx(math.pi * (2 * 10) * 30)) == 1

    def test_cut_volume(self):
        """Creates an ExtrudeCircleShape with another ExtrudeCircleShape cut out
        and checks that the volume is correct."""

        shape_with_cut = ExtrudeCircleShape(
            points=[(30, 0)], radius=20, distance=40,
            cut=self.test_shape
        )

        assert shape_with_cut.volume() == pytest.approx(
            (math.pi * (20**2) * 40) - (math.pi * (10**2) * 30)
        )

    def test_intersect_volume(self):
        """Creates ExtrudeCircleShapes with other ExtrudeCircleShapes
        intersected and checks that their volumes are correct."""

        intersect_shape = ExtrudeCircleShape(
            points=[(30, 0)], radius=5, distance=50)

        intersected_shape = ExtrudeCircleShape(
            points=[(30, 0)], radius=10, distance=50,
            intersect=[self.test_shape, intersect_shape]
        )

        assert intersected_shape.volume() == pytest.approx(math.pi * 5**2 * 30)

    def test_rotation_angle(self):
        """Creates an ExtrudeCircleShape with a rotation_angle < 360 and checks
        that the correct cut is performed and the volume is correct."""

        self.test_shape.azimuth_placement_angle = [45, 135, 225, 315]
        test_volume = self.test_shape.volume()
        self.test_shape.rotation_angle = 180
        assert self.test_shape.volume() == pytest.approx(test_volume * 0.5)

    def test_extrude_both(self):
        """Creates an ExtrudeCircleShape with extrude_both = True and False
        and checks that the volumes are correct."""

        test_volume_extrude_both = self.test_shape.volume()
        self.test_shape.extrude_both = False
        assert self.test_shape.volume() == pytest.approx(
            test_volume_extrude_both)

    def test_export_stp(self):
        """Exports and stp file with mode = solid and wire and checks
        that the outputs exist and relative file sizes are correct."""

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

        self.test_shape.export_stp('test_solid.stp', mode='solid')
        self.test_shape.export_stp('test_solid2.stp')
        self.test_shape.export_stp('test_wire.stp', mode='wire')

        assert Path("test_solid.stp").exists() is True
        assert Path("test_solid2.stp").exists() is True
        assert Path("test_wire.stp").exists() is True

        assert Path("test_solid.stp").stat().st_size == \
            Path("test_solid2.stp").stat().st_size
        assert Path("test_wire.stp").stat().st_size < \
            Path("test_solid2.stp").stat().st_size

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")


if __name__ == "__main__":
    unittest.main()




import paramak
import numpy as np
import cadquery as cq


class SegmentedBlanketBallReactor(paramak.BallReactor):
    """Creates geometry for a single ball reactor with a single divertor
    including a plasma, cylindrical center column shielding, square toroidal
    field coils. There is no inboard breeder blanket on this ball reactor like
    most spherical reactors.

    Arguments:
        gap_between_blankets (float): the distance between adjacent blanket
            segments,
        number_of_blanket_segments (int): the number of segments to divide the
            blanket up into. This for a full 360 degrees rotation
        blanket_fillet_radius (float): the fillet radius to apply to the
            interface between the firstwall and th breeder zone. Set to 0 for
            no fillet. Defaults to 10.0.
    """

    def __init__(
            self,
            gap_between_blankets,
            number_of_blanket_segments,
            blanket_fillet_radius: float = 10.0,
            **kwargs
    ):

        self.gap_between_blankets = gap_between_blankets
        self.number_of_blanket_segments = number_of_blanket_segments
        self.blanket_fillet_radius = blanket_fillet_radius

        super().__init__(**kwargs)

        # adds self.input_variable_names from the Reactor class
        self.input_variable_names = self.input_variable_names + [
            'gap_between_blankets',
            'number_of_blanket_segments',
            'blanket_fillet_radius'
        ]

    @property
    def gap_between_blankets(self):
        return self._gap_between_blankets

    @gap_between_blankets.setter
    def gap_between_blankets(self, value):
        """Sets the SegmentedBlanketBallReactor.gap_between_blankets
        attribute which controls the horitzonal distance between blanket
        segments."""
        if isinstance(value, (float, int)) and value > 0:
            self._gap_between_blankets = float(value)
        else:
            raise ValueError(
                "gap_between_blankets but be a positive value float")

    @property
    def number_of_blanket_segments(self):
        """Sets the SegmentedBlanketBallReactor.number_of_blanket_segments
        attribute which controls the number of blanket segments."""
        return self._number_of_blanket_segments

    @number_of_blanket_segments.setter
    def number_of_blanket_segments(self, value):
        if isinstance(value, int) and value > 2:
            self._number_of_blanket_segments = value
        else:
            raise ValueError(
                "number_of_blanket_segments but be an int greater than 2")

    def _make_blankets_layers(self):
        super()._make_blankets_layers()
        azimuth_placement_angles = np.linspace(
            0, 360, self.number_of_blanket_segments, endpoint=False)
        thin_cutter = paramak.BlanketCutterStar(
            distance=self.gap_between_blankets,
            azimuth_placement_angle=azimuth_placement_angles)

        thick_cutter = paramak.BlanketCutterStar(
            distance=self.gap_between_blankets +
            2 * self.firstwall_radial_thickness,
            azimuth_placement_angle=azimuth_placement_angles)

        self._blanket.cut = [self._center_column_cutter, thick_cutter]

        if self.blanket_fillet_radius != 0:
            # tried firstwall start radius here already
            x = self.major_radius + 1
            front_face_b = self._blanket.solid.faces(
                cq.NearestToPointSelector((0, x, 0)))
            front_edge_b = front_face_b.edges(
                cq.NearestToPointSelector((0, x, 0)))
            front_edge_length_b = front_edge_b.val().Length()
            self._blanket.solid = self._blanket.solid.edges(
                paramak.EdgeLengthSelector(front_edge_length_b)).fillet(
                self.blanket_fillet_radius)
        self._firstwall.thickness += self.blanket_radial_thickness
        self._firstwall.cut = [
            self._center_column_cutter,
            thin_cutter,
            self._blanket]

        # TODO this segfaults at the moment but works as an opperation on the
        # reactor after construction in jupyter
        # tried different x values and (0, x, 0)
        # noticed that it much quicker as a post process so perhaps some
        # unwanted looping is happening
        # if self.blanket_fillet_radius != 0:
        #     x = self.major_radius # tried firstwall start radius here already
        #     front_face = \
        #       self._firstwall.solid.faces(
        #           cq.NearestToPointSelector((x, 0, 0)))
        #     print('found front face')
        #     front_edge = front_face.edges(
        #           cq.NearestToPointSelector((x, 0, 0)))
        #     print('found front edge')
        #     front_edge_length = front_edge.val().Length()
        #     print('found front edge length', front_edge_length)
        #     self._firstwall.solid = self._firstwall.solid.edges(
        #         paramak.EdgeLengthSelector(front_edge_length)).fillet(self.blanket_fillet_radius)
        # print('finished')

        return [self._firstwall, self._blanket, self._blanket_rear_wall]




import paramak


class SingleNullBallReactor(paramak.BallReactor):
    """Creates geometry for a single ball reactor with a single divertor
    including a plasma, cylindrical center column shielding, square toroidal
    field coils. There is no inboard breeder blanket on this ball reactor like
    most spherical reactors.

    Arguments:
        divertor_position (str): Defaults to "upper".
    """

    def __init__(
        self,
        divertor_position: str = "upper",
        **kwargs
    ):

        super().__init__(divertor_position=divertor_position, **kwargs)



import os
import unittest
from pathlib import Path

import paramak


class TestCylinderReactor(unittest.TestCase):
    """Tests the CylinderReactor functionality"""

    def setUp(self):
        self.test_reactor = paramak.CylinderReactor(
            inner_blanket_radius=100,
            blanket_thickness=60,
            blanket_height=500,
            lower_blanket_thickness=50,
            upper_blanket_thickness=40,
            blanket_vv_gap=20,
            upper_vv_thickness=10,
            vv_thickness=10,
            lower_vv_thickness=10,
            rotation_angle=180,
        )

    def test_input_variable_names(self):
        """tests that the number of inputs variables is correct"""

        assert len(self.test_reactor.input_variables.keys()) == 13
        assert len(self.test_reactor.input_variable_names) == 13

    def test_stp_file_creation(self):
        """Creates a reactor with exports the step files and check they exist"""

        os.system('rm *.stp')
        self.test_reactor.export_stp(filename='cylinder.stp')
        assert Path('cylinder.stp').is_file()

    def test_html_file_creation(self):
        """Creates a reactor with exports the step files and check they exist"""

        os.system('rm *.html')
        self.test_reactor.export_html_3d('cylinder.html')
        assert Path('cylinder.html').is_file()

    def test_export_brep(self):
        """Exports a brep file and checks that the output exist"""

        os.system("rm test_reactor.brep")

        self.test_reactor.export_brep(filename='test_reactor.brep')

        assert Path("test_reactor.brep").exists() is True

        os.system("rm test_reactor.brep")

    def test_export_brep_without_extention(self):
        """Exports a brep file without the extention and checks that the
        output exist"""

        def missing_extention():

            self.test_reactor.export_brep(filename='test_reactor_missing')

        self.assertRaises(
            ValueError,
            missing_extention
        )




import os
import unittest
from pathlib import Path

import pytest
from paramak import RotateStraightShape, SweepSplineShape


class TestSweepSplineShape(unittest.TestCase):

    def setUp(self):
        self.test_shape = SweepSplineShape(
            points=[(-10, 10), (10, 10), (10, -10), (-10, -10)],
            path_points=[(50, 0), (30, 50), (70, 100), (50, 150)]
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a SweepSplineShape are correct."""

        # assert self.test_shape.rotation_angle == 360
        assert self.test_shape.azimuth_placement_angle == 0
        assert self.test_shape.workplane == "XY"
        assert self.test_shape.path_workplane == "XZ"
        assert self.test_shape.force_cross_section == False

    def test_solid_construction_workplane(self):
        """Checks that SweepSplineShapes can be created in different workplanes."""

        self.test_shape.workplane = "YZ"
        self.test_shape.path_workplane = "YX"
        assert self.test_shape.solid is not None

        self.test_shape.workplane = "XZ"
        self.test_shape.path_workplane = "XY"
        assert self.test_shape.solid is not None

    def test_relative_shape_volume_points(self):
        """Creates two SweepSplineShapes and checks that their relative volumes
        are correct."""

        self.test_shape.points = [(-20, 20), (20, 20), (20, -20), (-20, -20)]
        test_volume = self.test_shape.volume()
        self.test_shape.points = [(-10, 10), (10, 10), (10, -10), (-10, -10)]
        assert self.test_shape.volume() == pytest.approx(
            test_volume * 0.25, rel=0.01)

    def test_relative_shape_volume_azimuthal_placement(self):
        """Creates two SweepSplineShapes and checks that their relative volumes
        are correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.azimuth_placement_angle = [0, 90, 180, 270]
        assert self.test_shape.volume() == pytest.approx(
            test_volume * 4, rel=0.01)

    def test_force_cross_section(self):
        """Checks that a SweepSplineShape with the same cross-section at each path_point
        is created when force_cross_section = True."""

        self.test_shape.force_cross_section = True

        test_area = round(min(self.test_shape.areas))

        assert self.test_shape.areas.count(
            pytest.approx(test_area, rel=0.01)) == 2

        cutting_shape = RotateStraightShape(
            points=[(0, 50), (0, 200), (100, 200), (100, 50)]
        )
        self.test_shape.cut = cutting_shape

        assert self.test_shape.areas.count(
            pytest.approx(test_area, rel=0.01)) == 2

        cutting_shape.points = [(0, 100), (0, 200), (100, 200), (100, 100)]
        self.test_shape.cut = cutting_shape

        assert self.test_shape.areas.count(
            pytest.approx(test_area, rel=0.01)) == 2

    def test_force_cross_section_volume(self):
        """Checks that a SweepSplineShape with a larger volume is created when
        force_cross_section = True than when force_cross_section = False."""

        test_volume = self.test_shape.volume()
        self.test_shape.force_cross_section = True
        assert self.test_shape.volume() > test_volume

    def test_surface_count(self):
        """Creates a SweepSplineShape and checks that it has the correct number
        of surfaces."""

        assert len(self.test_shape.areas) == 3
        assert len(set(round(i) for i in self.test_shape.areas)) == 2

    def test_export_stp(self):
        """Exports and stp file with mode = solid and wire and checks
        that the outputs exist and relative file sizes are correct."""

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

        self.test_shape.export_stp('test_solid.stp', mode='solid')
        self.test_shape.export_stp('test_solid2.stp')
        self.test_shape.export_stp('test_wire.stp', mode='wire')

        assert Path("test_solid.stp").exists() is True
        assert Path("test_solid2.stp").exists() is True
        assert Path("test_wire.stp").exists() is True

        assert Path("test_solid.stp").stat().st_size == \
            Path("test_solid2.stp").stat().st_size
        assert Path("test_wire.stp").stat().st_size < \
            Path("test_solid2.stp").stat().st_size

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

    def test_incorrect_points_input(self):
        """Checks that an error is raised when the points are input with the
        connection"""

        def incorrect_points_definition():
            self.test_shape.points = [
                (10, 10, 'spline'),
                (10, 30, 'spline'),
                (30, 30, 'spline'),
                (30, 10, 'spline')
            ]

        self.assertRaises(
            ValueError,
            incorrect_points_definition
        )


if __name__ == "__main__":
    unittest.main()




import json
import os
import unittest
from pathlib import Path

import paramak
import pytest

cwd = os.getcwd()

"These tests require a visual front end which is not well suported on docker based CI systems"


class TestLocalExamples(unittest.TestCase):
    def test_make_collarge(self):
        """ Runs the example and checks the output files are produced"""
        os.chdir(Path(cwd))
        os.chdir(Path("examples"))
        output_filenames = [
            "output_collarge/1.png",
            "output_collarge/2.png",
            "output_collarge/3.png",
            "output_collarge/4.png",
            "output_collarge/5.png",
            "output_collarge/6.png",
            "output_collarge/7.png",
            "output_collarge/8.png",
            "output_collarge/9.png",
            "output_collarge/combine_images1.sh",
            "output_collarge/combine_images2.sh",
            "output_collarge/paramak_array1.svg",
            "output_collarge/paramak_array2.svg",
        ]
        for output_filename in output_filenames:
            os.system("rm " + output_filename)
        os.system("python make_collarge.py")
        for output_filename in output_filenames:
            assert Path(output_filename).exists()
            os.system("rm " + output_filename)

    def test_make_paramak_animation(self):
        """ Runs the example and checks the output files are produced"""
        os.chdir(Path(cwd))
        os.chdir(Path("examples"))
        output_filenames = [
            "output_for_animation_2d/0.png",
            "output_for_animation_2d/1.png",
            "output_for_animation_2d/2.png",
            "output_for_animation_3d/0.png",
            "output_for_animation_3d/1.png",
            "output_for_animation_3d/2.png",
            "2d.gif",
            "3d.gif",
        ]
        for output_filename in output_filenames:
            os.system("rm " + output_filename)
        os.system("python make_animation.py -n 3")
        for output_filename in output_filenames:
            assert Path(output_filename).exists()
            os.system("rm " + output_filename)

    def test_export_3d_image(self):
        """checks that export_3d_image() exports png files with the correct suffix"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20), (20, 0)]
        )
        test_shape.rotation_angle = 360
        os.system("rm filename.png")
        test_shape.export_3d_image("filename")
        assert Path("filename.png").exists() is True
        os.system("rm filename.png")
        test_shape.export_3d_image("filename.png")
        assert Path("filename.png").exists() is True
        os.system("rm filename.png")

    def test_neutronics_cell_tally(self):
        """ Runs the neutronics example and checks the TBR"""
        os.chdir(Path(cwd))
        os.chdir(Path("examples/neutronics"))
        output_filename = "simulation_result.json"
        os.system("rm " + output_filename)
        os.system("python make_simple_neutronics_model.py")
        with open(output_filename) as json_file:
            data = json.load(json_file)
        assert data["TBR"] == pytest.approx(0.456, abs=0.01)
        os.system("rm " + output_filename)


if __name__ == "__main__":
    unittest.main()




import math
from typing import Optional, Tuple

from paramak import ExtrudeStraightShape


class HexagonPin(ExtrudeStraightShape):
    """Creates an extruded hexagon by a provided distance about a center point.

    Args:
        length_of_side: the length of one side of the hexagon (mm).
        distance: extruded distance along the y-direction (mm).
        center_point: the center of the hexagon on the x-z plane (mm).
        name: defaults to "hexagon_pin".
    """

    def __init__(
        self,
        length_of_side: float,
        distance: float,
        center_point: Tuple[float, float] = (0, 0),
        name: Optional[str] = "hexagon_pin",
        **kwargs
    ) -> None:

        super().__init__(
            name=name,
            distance=distance,
            **kwargs
        )

        self.center_point = center_point
        self.length_of_side = length_of_side

    @property
    def center_point(self):
        return self._center_point

    @center_point.setter
    def center_point(self, center_point):
        self._center_point = center_point

    @property
    def length_of_side(self):
        return self._length_of_side

    @length_of_side.setter
    def length_of_side(self, length_of_side):
        self._length_of_side = length_of_side

    def find_points(self):
        """Finds the XZ points joined by straight connections that describe
        the 2D profile of the hexagon faced shape."""

        # map of points
        #     p3 ---- p2
        #    -           -
        #   -             -
        #  -               -
        # p4                p1
        #  -               -
        #   -             -
        #    -          -
        #     p5 ---- p6

        points = []
        for i in range(6):
            point = (self.length_of_side * math.cos(math.pi / 3 * i) +
                     self.center_point[0],
                     self.length_of_side * math.sin(math.pi / 3 * i) +
                     self.center_point[1])
            points.append(point)

        self.points = points




from typing import Optional, Tuple

from paramak import ExtrudeMixedShape


class ExtrudeStraightShape(ExtrudeMixedShape):
    """Extrudes a 3d CadQuery solid from points connected with straight lines.

    Args:
        distance: the extrusion distance to use (cm units if used for
            neutronics)
    """

    def __init__(
        self,
        distance: float,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (1.0, 0.498, 0.0),
        **kwargs
    ):

        super().__init__(
            distance=distance,
            connection_type="straight",
            color=color,
            **kwargs
        )




import os
import unittest
from pathlib import Path

import pytest
from paramak import ExtrudeMixedShape


class TestExtrudeMixedShape(unittest.TestCase):

    def setUp(self):
        self.test_shape = ExtrudeMixedShape(
            points=[
                (50, 0, "straight"),
                (50, 50, "spline"),
                (60, 70, "spline"),
                (70, 50, "circle"),
                (60, 25, "circle"),
                (70, 0, "straight")
            ],
            distance=50
        )

        self.test_shape_2 = ExtrudeMixedShape(
            distance=1,
            points=[
                (100, 0, "straight"),
                (200, 0, "circle"),
                (250, 50, "circle"),
                (200, 100, "straight"),
                (150, 100, "straight"),
                (140, 75, "straight"),
                (110, 45, "straight"),
            ]
        )

        self.test_shape_3 = ExtrudeMixedShape(
            distance=180,
            points=[
                (100, 0, "straight"),
                (200, 0, "circle"),
                (250, 50, "circle"),
                (200, 100, "straight"),
                (150, 100, "straight"),
                (140, 75, "circle"),
                (110, 45, "circle"),
            ]
        )

    def test_default_parameters(self):
        """Checks that the default parameters of an ExtrudeMixedShape are
        correct."""

        assert self.test_shape.rotation_angle == 360
        assert self.test_shape.azimuth_placement_angle == 0

    def test_absolute_shape_volume(self):
        """Creates an ExtrudeMixedShape and checks that the volume is
        correct."""

        assert self.test_shape.volume() > 20 * 20 * 30

    def test_relative_shape_volume(self):
        """Creates two ExtrudeMixedShapes and checks that their relative volumes
        are correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.azimuth_placement_angle = [0, 180]

        assert self.test_shape.volume() == pytest.approx(
            test_volume * 2, rel=0.01)

    def test_shape_face_areas(self):
        """Creates an ExtrudeMixedShape and checks that the face areas are
        expected."""

        self.test_shape.extrude_both = False
        assert len(self.test_shape.areas) == 6
        assert len(set([round(i) for i in self.test_shape.areas])) == 5

    def test_cut_volume(self):
        """Creates an ExtrudeMixedShape with another ExtrudeMixedShape cut out
        and checks that the volume is correct."""

        inner_shape = ExtrudeMixedShape(
            points=[
                (5, 5, "straight"),
                (5, 10, "spline"),
                (10, 10, "spline"),
                (10, 5, "spline"),
            ],
            distance=30,
        )

        outer_shape = ExtrudeMixedShape(
            points=[
                (3, 3, "straight"),
                (3, 12, "spline"),
                (12, 12, "spline"),
                (12, 3, "spline"),
            ],
            distance=30,
        )

        outer_shape_with_cut = ExtrudeMixedShape(
            points=[
                (3, 3, "straight"),
                (3, 12, "spline"),
                (12, 12, "spline"),
                (12, 3, "spline"),
            ],
            cut=inner_shape,
            distance=30,
        )

        assert inner_shape.volume() == pytest.approx(1068, abs=2)
        assert outer_shape.volume() == pytest.approx(3462, abs=2)
        assert outer_shape_with_cut.volume() == pytest.approx(2394, abs=2)

    def test_export_stp_extension(self):
        """Creates an ExtrudeMixedShape and checks that a stp file of the shape
        can be exported with the correct suffix using the export_stp method."""

        os.system("rm filename.stp filename.step")
        self.test_shape.export_stp("filename.stp")
        self.test_shape.export_stp("filename.step")
        assert Path("filename.stp").exists() is True
        assert Path("filename.step").exists() is True
        os.system("rm filename.stp filename.step")

    def test_export_stl(self):
        """Creates a ExtrudeMixedShape and checks that a stl file of the shape
        can be exported with the correct suffix using the export_stl method."""

        os.system("rm filename.stl")
        self.test_shape.export_stl("filename.stl")
        assert Path("filename.stl").exists() is True
        os.system("rm filename.stl")

    def test_rotation_angle(self):
        """Creates an ExtrudeMixedShape with a rotation_angle < 360 and checks
        that the correct cut is performed and the volume is correct."""

        self.test_shape.azimuth_placement_angle = [45, 135, 225, 315]
        test_volume = self.test_shape.volume()
        self.test_shape.rotation_angle = 180
        assert self.test_shape.volume() == pytest.approx(
            test_volume * 0.5, rel=0.01)

    def test_extrude_both(self):
        """Creates an ExtrudeMixedShape with extrude_both = True and False and
        checks that the volumes are correct."""

        test_volume_extrude_both = self.test_shape.volume()
        self.test_shape.extrude_both = False
        assert self.test_shape.volume() == pytest.approx(
            test_volume_extrude_both)

    def test_export_stp(self):
        """Exports and stp file with mode = solid and wire and checks
        that the outputs exist and relative file sizes are correct."""

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

        self.test_shape.export_stp('test_solid.stp', mode='solid')
        self.test_shape.export_stp('test_solid2.stp')
        self.test_shape.export_stp('test_wire.stp', mode='wire')

        assert Path("test_solid.stp").exists() is True
        assert Path("test_solid2.stp").exists() is True
        assert Path("test_wire.stp").exists() is True

        assert pytest.approx(Path("test_solid.stp").stat().st_size, rel=1) == \
            Path("test_solid2.stp").stat().st_size
        assert Path("test_wire.stp").stat().st_size < \
            Path("test_solid2.stp").stat().st_size

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

    def test_convert_all_circle_points_change_to_splines(self):
        """creates a ExtrudeMixedShape with one circular edges and converts
        them to spline edges. Checks the new edges have been correctly
        replaced with splines"""

        assert len(self.test_shape_2.processed_points) == 8
        self.test_shape_2.convert_all_circle_connections_to_splines()
        assert len(self.test_shape_2.processed_points) > 8
        assert self.test_shape_2.processed_points[0] == (100, 0, "straight")
        assert self.test_shape_2.processed_points[1] == (200, 0, 'spline')

        # last point is the same as the first point
        assert self.test_shape_2.processed_points[-1] == (100, 0, "straight")
        assert self.test_shape_2.processed_points[-2] == (110, 45, "straight")
        assert self.test_shape_2.processed_points[-3] == (140, 75, "straight")
        assert self.test_shape_2.processed_points[-4] == (150, 100, "straight")
        assert self.test_shape_2.processed_points[-5] == (200, 100, "straight")

        for point in self.test_shape_2.processed_points[1:len(
                self.test_shape_2.points) - 5]:
            assert point[2] == 'spline'

    def test_convert_circles_to_splines_volume(self):
        """creates a RotateMixedShape with a circular edge and converts the
        edge to a spline edges. Checks the new shape has approximately the same
        volume as the original shape (with circles)"""

        original_volume = self.test_shape_2.volume()
        self.test_shape_2.convert_all_circle_connections_to_splines()
        new_volume = self.test_shape_2.volume()

        assert pytest.approx(new_volume, rel=0.000001) == original_volume

        original_volume = self.test_shape_3.volume()
        self.test_shape_3.convert_all_circle_connections_to_splines()
        new_volume = self.test_shape_3.volume()

        assert pytest.approx(new_volume, rel=0.000001) == original_volume


if __name__ == "__main__":
    unittest.main()




from typing import Optional, Tuple
import cadquery as cq
import numpy as np
from paramak import ExtrudeStraightShape
from paramak.utils import calculate_wedge_cut


class ToroidalFieldCoilRectangle(ExtrudeStraightShape):
    """Creates a rectangular shaped toroidal field coil.

    Args:
        horizontal_start_point: the (x,z) coordinates of the inner upper
            point (cm).
        vertical_mid_point: the (x,z) coordinates of the mid point of the
            vertical section (cm).
        thickness: the thickness of the toroidal field coil.
        distance: the extrusion distance.
        number_of_coils: the number of tf coils. This changes by the
            azimuth_placement_angle dividing up 360 degrees by the number of
            coils.
        with_inner_leg: include the inner tf leg. Defaults to True.
    """

    def __init__(
        self,
        horizontal_start_point: Tuple[float, float],
        vertical_mid_point: Tuple[float, float],
        thickness: float,
        distance: float,
        number_of_coils: int,
        with_inner_leg: Optional[bool] = True,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (0., 0., 1.),
        **kwargs
    ) -> None:

        super().__init__(
            distance=distance,
            color=color,
            **kwargs
        )

        self.horizontal_start_point = horizontal_start_point
        self.vertical_mid_point = vertical_mid_point
        self.thickness = thickness
        self.distance = distance
        self.number_of_coils = number_of_coils
        self.with_inner_leg = with_inner_leg

    @property
    def azimuth_placement_angle(self):
        self.find_azimuth_placement_angle()
        return self._azimuth_placement_angle

    @azimuth_placement_angle.setter
    def azimuth_placement_angle(self, value):
        self._azimuth_placement_angle = value

    def find_points(self):
        """Finds the XZ points joined by straight connections that describe
        the 2D profile of the poloidal field coil shape."""

        if self.horizontal_start_point[0] >= self.vertical_mid_point[0]:
            raise ValueError(
                'horizontal_start_point x should be smaller than the \
                    vertical_mid_point x value')
        if self.vertical_mid_point[1] >= self.horizontal_start_point[1]:
            raise ValueError(
                'vertical_mid_point y value should be smaller than the \
                    horizontal_start_point y value')

        points = [
            self.horizontal_start_point,  # connection point
            # connection point
            (self.horizontal_start_point[0] +
             self.thickness, self.horizontal_start_point[1]),
            (self.vertical_mid_point[0], self.horizontal_start_point[1]),
            (self.vertical_mid_point[0], -self.horizontal_start_point[1]),
            # connection point
            (self.horizontal_start_point[0] +
             self.thickness, -
             self.horizontal_start_point[1]),
            # connection point
            (self.horizontal_start_point[0], -self.horizontal_start_point[1]),
            (self.horizontal_start_point[0], -
             (self.horizontal_start_point[1] +
                self.thickness)),
            (self.vertical_mid_point[0] +
             self.thickness, -
             (self.horizontal_start_point[1] +
                self.thickness)),
            (self.vertical_mid_point[0] + self.thickness,
             self.horizontal_start_point[1] + self.thickness),
            (self.horizontal_start_point[0],
             self.horizontal_start_point[1] + self.thickness),
        ]

        self.inner_leg_connection_points = [
            points[0], points[1], points[4], points[5]]

        self.points = points

    def find_azimuth_placement_angle(self):
        """Calculates the azimuth placement angles based on the number of tf
        coils"""

        angles = list(
            np.linspace(
                0,
                360,
                self.number_of_coils,
                endpoint=False))

        self.azimuth_placement_angle = angles

    def create_solid(self):
        """Creates a 3d solid using points with straight edges.

        Returns:
           A CadQuery solid: A 3D solid volume
        """

        # Creates a cadquery solid from points and revolves
        points_without_connections = [p[:2] for p in self.points]
        solid = (
            cq.Workplane(self.workplane)
            .polyline(points_without_connections)
        )

        wire = solid.close()

        self.wire = wire

        solid = wire.extrude(distance=-self.distance / 2.0, both=True)

        solid = self.rotate_solid(solid)

        cutting_wedge = calculate_wedge_cut(self)
        solid = self.perform_boolean_operations(solid, wedge_cut=cutting_wedge)

        if self.with_inner_leg is True:
            inner_leg_solid = cq.Workplane(self.workplane)
            inner_leg_solid = inner_leg_solid.polyline(
                self.inner_leg_connection_points)
            inner_leg_solid = inner_leg_solid.close().extrude(
                distance=-self.distance / 2.0, both=True)

            inner_leg_solid = self.rotate_solid(inner_leg_solid)
            inner_leg_solid = self.perform_boolean_operations(
                inner_leg_solid, wedge_cut=cutting_wedge)

            solid = cq.Compound.makeCompound(
                [a.val() for a in [inner_leg_solid, solid]]
            )

        self.solid = solid   # not necessarily required as set in boolean_operations

        return solid




from typing import Optional
from paramak import ExtrudeStraightShape


class BlanketCutterStar(ExtrudeStraightShape):
    """Creates an extruded shape with a rectangular section that is used to cut
    other components (eg. blankets and firstwalls) in order to create banana
    style blanket segments. Typically used to divide a blanket into vertical
    sections with a fixed gap between each section.

    Args:
        distance: extruded distance (cm) of the cutter which translates to being
            the gap size between blankets when the cutter is used to segment
            blankets.
        height: height (cm) of the port. Defaults to 2000.0.
        width: width (cm) of the port. Defaults to 2000.0.
        azimuth_placement_angle (list or float, optional): Defaults to
            [0., 36., 72., 108., 144., 180., 216., 252., 288., 324.]
        name: defaults to "blanket_cutter_star".
    """

    def __init__(
            self,
            distance: float,
            height: Optional[float] = 2000.,
            width: Optional[float] = 2000.,
            azimuth_placement_angle: Optional[float] = [
                0.,
                36.,
                72.,
                108.,
                144.,
                180.,
                216.,
                252.,
                288.,
                324.],
            name: Optional[str] = "blanket_cutter_star",
            **kwargs) -> None:

        super().__init__(
            extrude_both=True,
            name=name,
            azimuth_placement_angle=azimuth_placement_angle,
            distance=distance,
            **kwargs
        )

        self.azimuth_placement_angle = azimuth_placement_angle
        self.height = height
        self.width = width
        self.distance = distance

    def find_points(self):

        points = [
            (0, -self.height / 2),
            (self.width, -self.height / 2),
            (self.width, self.height / 2),
            (0, self.height / 2),
        ]
        self.points = points




import os
import unittest
from pathlib import Path

import pytest
from paramak import RotateStraightShape, SweepStraightShape


class TestSweepStraightShape(unittest.TestCase):

    def setUp(self):
        self.test_shape = SweepStraightShape(
            points=[(-10, 10), (10, 10), (10, -10), (-10, -10)],
            path_points=[(50, 0), (30, 50), (70, 100), (50, 150)]
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a SweepStraightShape are
        correct."""

        # assert self.test_shape.rotation_angle == 360
        assert self.test_shape.azimuth_placement_angle == 0
        assert self.test_shape.workplane == "XY"
        assert self.test_shape.path_workplane == "XZ"
        assert self.test_shape.force_cross_section == False

    def test_absolute_shape_volume(self):
        """Creates a SweepStraightShape and checks that the volume is
        correct."""

        self.test_shape.path_points = [(50, 0), (50, 50), (50, 100)]
        assert self.test_shape.solid is not None
        assert self.test_shape.volume() == pytest.approx(20 * 20 * 100)

    def test_relative_shape_volume(self):
        """Creates two SweepStraightShapes and checks that their relative
        volumes are correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.points = [(-20, 20), (20, 20), (20, -20), (-20, -20)]
        assert self.test_shape.volume() == pytest.approx(test_volume * 4)

    def test_relative_shape_volume_again(self):
        """Creates two SweepStraightShapes and checks that their relative volumes
        are correct."""

        test_volume = self.test_shape.volume()
        self.test_shape.azimuth_placement_angle = [0, 90, 180, 270]
        assert self.test_shape.volume() == pytest.approx(test_volume * 4)

    def test_force_cross_section(self):
        """Checks that a SweepStraightShape with the same cross-section at each path_point
        is created when force_cross_section = True."""

        self.test_shape.force_cross_section = True

        assert self.test_shape.areas.count(pytest.approx(400, rel=0.01)) == 2

        cutting_shape = RotateStraightShape(
            points=[(0, 50), (0, 200), (100, 200), (100, 50)],
        )
        self.test_shape.cut = cutting_shape

        assert self.test_shape.areas.count(pytest.approx(400, rel=0.01)) == 2

        cutting_shape.points = [(0, 100), (0, 200), (100, 200), (100, 100)]
        self.test_shape.cut = cutting_shape

        assert self.test_shape.areas.count(pytest.approx(400, rel=0.01)) == 2

    def test_force_cross_section_volume(self):
        """Checks that a SweepStraightShape with a larger volume is created when
        force_cross_section = True than when force_cross_section = False."""

        test_volume = self.test_shape.volume()
        self.test_shape.force_cross_section = True
        assert self.test_shape.volume() > test_volume

    def test_surface_count(self):
        """Creates a SweepStraightShape and checks that it has the correct number
        of surfaces."""

        assert len(self.test_shape.areas) == 6
        assert len(set(round(i) for i in self.test_shape.areas)) == 3

    def test_export_stp(self):
        """Exports and stp file with mode = solid and wire and checks
        that the outputs exist and relative file sizes are correct."""

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

        self.test_shape.export_stp('test_solid.stp', mode='solid')
        self.test_shape.export_stp('test_solid2.stp')
        self.test_shape.export_stp('test_wire.stp', mode='wire')

        assert Path("test_solid.stp").exists() is True
        assert Path("test_solid2.stp").exists() is True
        assert Path("test_wire.stp").exists() is True

        assert Path("test_solid.stp").stat().st_size == \
            Path("test_solid2.stp").stat().st_size
        assert Path("test_wire.stp").stat().st_size < \
            Path("test_solid2.stp").stat().st_size

        os.system("rm test_solid.stp test_solid2.stp test_wire.stp")

    def test_export_brep(self):
        """Exports a brep file and checks that the output exist"""

        os.system("rm test_solid.brep")

        self.test_shape.export_brep(filename='test_solid.brep')

        assert Path("test_solid.brep").exists() is True

        os.system("rm test_solid.brep")

    def test_export_brep_without_extention(self):
        """Exports a brep file without the extention and checks that the
        output exist"""

        def missing_extention():

            self.test_shape.export_brep(filename='test_solid_missing')

        self.assertRaises(
            ValueError,
            missing_extention
        )

    def test_incorrect_points_input(self):
        """Checks that an error is raised when the points are input with the
        connection"""

        def incorrect_points_definition():
            self.test_shape.points = [
                (10, 10, 'straight'),
                (10, 30, 'straight'),
                (30, 30, 'straight'),
                (30, 10, 'straight')
            ]

        self.assertRaises(
            ValueError,
            incorrect_points_definition
        )


if __name__ == "__main__":
    unittest.main()




import json
import os
import unittest
from pathlib import Path

import cadquery as cq
import paramak
import pytest


class TestReactor(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)],
            name='test_shape'
        )

        self.test_shape2 = paramak.ExtrudeStraightShape(
            points=[(100, 100), (50, 100), (50, 50)], distance=20,
            name='test_shape2'
        )

        test_shape_3 = paramak.PoloidalFieldCoilSet(
            heights=[2, 2],
            widths=[3, 3],
            center_points=[(50, -100), (50, 100)]
        )

        self.test_reactor = paramak.Reactor([self.test_shape])

        self.test_reactor_2 = paramak.Reactor(
            [self.test_shape, self.test_shape2])

        # this reactor has a compound shape in the geometry
        self.test_reactor_3 = paramak.Reactor(
            [self.test_shape, test_shape_3])

    def test_reactor_export_stp(self):
        """Exports the reactor as seperate files and as a single file"""
        os.system('rm *.stp')
        self.test_reactor_2.export_stp(
            filename=['RotateStraightShape.stp', 'ExtrudeStraightShape.stp']
        )
        assert Path('RotateStraightShape.stp').is_file()
        assert Path('ExtrudeStraightShape.stp').is_file()
        self.test_reactor_2.export_stp(filename='single_file.stp', units='cm')
        assert Path('single_file.stp').is_file()

    def test_show_runs_without_error(self):
        """checks that the jupyter notebook (with cadquery addition) runs
        without error."""

        self.test_reactor.show()

    def test_show_runs_without_error_when_names_are_set(self):
        """checks that the jupyter notebook (with cadquery addition) runs
        without error even when the .name property is set"""

        self.test_reactor.shapes_and_components[0].name = 'test'
        self.test_reactor.show()

    def test_show_runs_without_error_when_compounds_are_used(self):
        """checks that the jupyter notebook (with cadquery addition) runs
        without error even when the .name property is set"""

        test_shape = paramak.PoloidalFieldCoilCaseSet(
            heights=[10, 20],
            widths=[10, 20],
            casing_thicknesses=10,
            center_points=[(100, 200), (400, 400)],
            name='test name'
        )

        test_reactor = paramak.Reactor([test_shape])
        test_reactor.show()

    def test_incorrect_graveyard_offset_too_small(self):

        def incorrect_graveyard_offset_too_small():
            """Set graveyard_offset as a negative number which should raise an error"""

            self.test_reactor.graveyard_offset = -3

        self.assertRaises(
            ValueError,
            incorrect_graveyard_offset_too_small
        )

    def test_incorrect_graveyard_offset_wrong_type(self):

        def incorrect_graveyard_offset_wrong_type():
            """Set graveyard_offset as a string which should raise an error"""

            self.test_reactor.graveyard_offset = 'coucou'

        self.assertRaises(
            TypeError,
            incorrect_graveyard_offset_wrong_type
        )

    def test_largest_dimension_setting_and_getting_using_largest_shapes(self):
        """Makes a neutronics model and checks the default largest_dimension
        and that largest_dimension changes with largest_shapes"""

        assert self.test_reactor.largest_dimension == 20.

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape2 = paramak.RotateStraightShape(
            points=[(0, 0), (0, 40), (40, 40)])

        test_reactor = paramak.Reactor([test_shape, test_shape2])
        assert test_reactor.largest_dimension == 40

        test_reactor.largest_shapes = [test_shape]
        assert test_reactor.largest_dimension == 20

    def test_make_sector_wedge(self):
        """Checks that the wedge is not made when rotation angle is 360"""
        sector_wedge = self.test_reactor.make_sector_wedge(
            height=100,
            radius=100,
            rotation_angle=360
        )
        assert sector_wedge is None

    def test_missing_filename_arg(self):

        def test_stl_filename_None():
            test_shape = paramak.ExtrudeCircleShape(
                points=[(20, 20)], radius=10, distance=10
            )
            my_reactor = paramak.Reactor([test_shape])
            my_reactor.export_stl()

        self.assertRaises(
            TypeError,
            test_stl_filename_None)

    def test_make_graveyard_accepts_offset_from_graveyard(self):
        """Creates a graveyard for a reactor and sets the graveyard_offset.
        Checks that the Reactor.graveyard_offset property is set"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)],
        )
        test_shape2 = paramak.RotateSplineShape(
            points=[(0, 0), (0, 20), (20, 20)],
        )
        test_shape.rotation_angle = 360
        test_reactor = paramak.Reactor([test_shape, test_shape2])
        test_reactor.graveyard_offset == 101
        graveyard = test_reactor.make_graveyard()
        assert graveyard.volume() > 0

    def test_reactor_creation_with_default_properties(self):
        """creates a Reactor object and checks that it has no default properties"""

        test_reactor = paramak.Reactor([])

        assert test_reactor is not None

    def test_adding_component_to_reactor(self):
        """creates a Reactor object and checks that shapes can be added to it"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_shape.create_solid()
        test_reactor = paramak.Reactor([])
        assert len(test_reactor.shapes_and_components) == 0
        test_reactor = paramak.Reactor([test_shape])
        assert len(test_reactor.shapes_and_components) == 1

    def test_graveyard_exists(self):
        """creates a Reactor object with one shape and checks that a graveyard
        can be produced using the make_graveyard method"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_shape.create_solid()
        test_reactor = paramak.Reactor([test_shape])
        test_reactor.make_graveyard()

        assert isinstance(test_reactor.graveyard, paramak.Shape)

    def test_graveyard_exists_solid_is_none(self):
        """creates a Reactor object with one shape and checks that a graveyard
        can be produced using the make_graveyard method when the solid
        attribute of the shape is None"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_shape.create_solid()
        test_reactor = paramak.Reactor([test_shape])
        test_reactor.shapes_and_components[0].solid = None
        test_reactor.make_graveyard()

        assert isinstance(test_reactor.graveyard, paramak.Shape)

    def test_export_graveyard(self):
        """creates a Reactor object with one shape and checks that a graveyard
        can be exported to a specified location using the make_graveyard method"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        os.system("rm my_graveyard.stp")
        os.system("rm graveyard.stp")
        test_reactor = paramak.Reactor([test_shape])

        test_reactor.export_stp_graveyard()
        test_reactor.export_stp_graveyard(filename="my_graveyard.stp")
        test_reactor.export_stp_graveyard(filename="my_graveyard_without_ext")

        for filepath in [
            "graveyard.stp",
            "my_graveyard.stp",
                "my_graveyard_without_ext.stp"]:
            assert Path(filepath).exists() is True
            os.system("rm " + filepath)

        assert test_reactor.graveyard is not None
        assert test_reactor.graveyard.__class__.__name__ == "HollowCube"

    def test_make_graveyard_offset(self):
        """checks that the graveyard can be exported with the correct default parameters
        and that these parameters can be changed"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        os.system("rm graveyard.stp")
        test_reactor = paramak.Reactor(
            [test_shape],
            graveyard_size=None,
            graveyard_offset=100)
        test_reactor.make_graveyard()

        graveyard_volume_1 = test_reactor.graveyard.volume()

        test_reactor.make_graveyard(graveyard_offset=50)
        assert test_reactor.graveyard.volume() < graveyard_volume_1
        graveyard_volume_2 = test_reactor.graveyard.volume()

        test_reactor.make_graveyard(graveyard_offset=200)
        assert test_reactor.graveyard.volume() > graveyard_volume_1
        assert test_reactor.graveyard.volume() > graveyard_volume_2

    def test_exported_stp_files_exist(self):
        """creates a Reactor object with one shape and checks that a stp file
        of the reactor can be exported to a specified location using the export_stp
        method"""

        os.system("rm test_reactor/*.stp")
        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360

        os.system("rm test_reactor/test_shape.stp")
        os.system("rm test_reactor/graveyard.stp")

        test_reactor = paramak.Reactor([test_shape])

        test_reactor.export_stp('test_reactor/test_shape.stp')

        assert Path("test_reactor/test_shape.stp").exists() is True

    def test_exported_stl_files_exist(self):
        """creates a Reactor object with one shape and checks that a stl file
        of the reactor can be exported to a specified location using the
        export_stl method"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        os.system("rm test_reactor/test_shape.stl")
        test_reactor = paramak.Reactor([test_shape])

        test_reactor.export_stl(filename='test_reactor/test_shape.stl')

        assert Path("test_reactor/test_shape.stl").exists() is True
        os.system("rm test_reactor/test_shape.stl")

    def test_exported_svg_files_exist(self):
        """Creates a Reactor object with one shape and checks that a svg file
        of the reactor can be exported to a specified location using the
        export_svg method."""

        os.system("rm test_svg_image.svg")

        self.test_reactor.export_svg("test_svg_image.svg")

        assert Path("test_svg_image.svg").exists() is True
        os.system("rm test_svg_image.svg")

    def test_exported_svg_files_exist_no_extension(self):
        """creates a Reactor object with one shape and checks that an svg file
        of the reactor can be exported to a specified location using the export_svg
        method"""

        os.system("rm test_svg_image.svg")

        self.test_reactor.export_svg("test_svg_image")

        assert Path("test_svg_image.svg").exists() is True
        os.system("rm test_svg_image.svg")

    def test_export_svg_options(self):
        """Exports the test reactor to an svg image and checks that a svg file
        can be exported with the various different export options"""

        os.system("rm *.svg")
        self.test_reactor.export_svg("r_width.svg", width=900)
        assert Path("r_width.svg").exists() is True
        self.test_reactor.export_svg("r_height.svg", height=900)
        assert Path("r_height.svg").exists() is True
        self.test_reactor.export_svg("r_marginLeft.svg", marginLeft=110)
        assert Path("r_marginLeft.svg").exists() is True
        self.test_reactor.export_svg("r_marginTop.svg", marginTop=110)
        assert Path("r_marginTop.svg").exists() is True
        self.test_reactor.export_svg("r_showAxes.svg", showAxes=True)
        assert Path("r_showAxes.svg").exists() is True
        self.test_reactor.export_svg(
            "r_projectionDir.svg", projectionDir=(-1, -1, -1))
        assert Path("r_projectionDir.svg").exists() is True
        self.test_reactor.export_svg(
            "r_strokeColor.svg", strokeColor=(
                42, 42, 42))
        assert Path("r_strokeColor.svg").exists() is True
        self.test_reactor.export_svg(
            "r_hiddenColor.svg", hiddenColor=(
                42, 42, 42))
        assert Path("r_hiddenColor.svg").exists() is True
        self.test_reactor.export_svg("r_showHidden.svg", showHidden=False)
        assert Path("r_showHidden.svg").exists() is True
        self.test_reactor.export_svg("r_strokeWidth1.svg", strokeWidth=None)
        assert Path("r_strokeWidth1.svg").exists() is True
        self.test_reactor.export_svg("r_strokeWidth2.svg", strokeWidth=10)
        assert Path("r_strokeWidth2.svg").exists() is True

    def test_export_2d_image(self):
        """Creates a Reactor object and checks that a png file of the reactor
        with the correct filename can be exported using the export_2D_image
        method."""

        os.system("rm 2D_test_image.png")
        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_reactor = paramak.Reactor([test_shape])
        returned_filename = test_reactor.export_2d_image(
            filename="2D_test_image.png")

        assert Path(returned_filename).exists() is True
        os.system("rm 2D_test_image.png")

    def test_export_2d_image_without_extension(self):
        """creates a Reactor object and checks that a png file of the reactor
        with the correct filename can be exported using the export_2d_image
        method"""

        os.system("rm 2d_test_image.png")
        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_reactor = paramak.Reactor([test_shape])
        returned_filename = test_reactor.export_2d_image(
            filename="2d_test_image")

        assert Path(returned_filename).exists() is True
        os.system("rm 2d_test_image.png")

    def test_export_html(self):
        """Creates a Reactor object and checks that a html file of the reactor
        with the correct filename can be exported using the export_html
        method."""

        os.system("rm test_html.html")
        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_reactor = paramak.Reactor([test_shape])
        test_reactor.export_html(filename="test_html.html")

        assert Path("test_html.html").exists() is True
        os.system("rm test_html.html")
        test_reactor.export_html(filename="test_html")

        assert Path("test_html.html").exists() is True
        os.system("rm test_html.html")

    def test_export_3d_html(self):
        """Checks the 3d html file is exported by the export_html_3d method
        with the correct filename"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_reactor = paramak.Reactor([test_shape])

        os.system("rm filename.html")
        filename = test_reactor.export_html_3d('filename.html')
        if filename is not None:
            assert Path("filename.html").exists() is True

    def test_tet_meshes_error(self):
        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_reactor = paramak.Reactor([test_shape])
        assert test_reactor.tet_meshes is not None

    def test_largest_dimension(self):
        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_shape.rotation_angle = 360
        test_reactor = paramak.Reactor([test_shape])
        assert pytest.approx(test_reactor.largest_dimension, rel=0.1 == 20)
        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (30, 20)])
        test_shape.rotation_angle = 360
        test_reactor = paramak.Reactor([test_shape])
        assert pytest.approx(test_reactor.largest_dimension, rel=0.1 == 30)

    def test_shapes_and_components(self):
        """Attempts to make a reactor with a single shape instead of a list of
        shapes which should raise a ValueError"""

        def incorrect_shapes_and_components():
            test_shape = paramak.RotateStraightShape(
                points=[(0, 0), (0, 20), (20, 20)])
            paramak.Reactor(test_shape)
        self.assertRaises(ValueError, incorrect_shapes_and_components)

    def test_graveyard_size_setting_type_checking(self):
        """Attempts to make a reactor with a graveyard_size that is an float
        which should raise a ValueError"""

        def incorrect_graveyard_size_type():
            test_shape = paramak.RotateStraightShape(
                points=[(0, 0), (0, 20), (20, 20)])
            paramak.Reactor([test_shape], graveyard_size='coucou')
        self.assertRaises(TypeError, incorrect_graveyard_size_type)

    def test_graveyard_size_setting_magnitude_checking(self):
        """Attempts to make a reactor with a graveyard_size that is an int
        which should raise a ValueError"""

        def incorrect_graveyard_size_size():
            test_shape = paramak.RotateStraightShape(
                points=[(0, 0), (0, 20), (20, 20)])
            paramak.Reactor([test_shape], graveyard_size=-10)
        self.assertRaises(ValueError, incorrect_graveyard_size_size)

    def test_graveyard_offset_setting_type_checking(self):
        """Attempts to make a reactor with a graveyard_offset that is an float
        which should raise a ValueError"""

        def incorrect_graveyard_offset_type():
            test_shape = paramak.RotateStraightShape(
                points=[(0, 0), (0, 20), (20, 20)])
            paramak.Reactor([test_shape], graveyard_offset='coucou')
        self.assertRaises(TypeError, incorrect_graveyard_offset_type)

    def test_graveyard_offset_setting_magnitude_checking(self):
        """Attempts to make a reactor with a graveyard_offset that is an int
        which should raise a ValueError"""

        def incorrect_graveyard_offset_size():
            test_shape = paramak.RotateStraightShape(
                points=[(0, 0), (0, 20), (20, 20)])
            paramak.Reactor([test_shape], graveyard_offset=-10)
        self.assertRaises(ValueError, incorrect_graveyard_offset_size)

    def test_graveyard_error(self):
        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        test_reactor = paramak.Reactor([test_shape])

        def str_graveyard_offset():
            test_reactor.graveyard_offset = 'coucou'
        self.assertRaises(TypeError, str_graveyard_offset)

        def negative_graveyard_offset():
            test_reactor.graveyard_offset = -2
        self.assertRaises(ValueError, negative_graveyard_offset)

        def list_graveyard_offset():
            test_reactor.graveyard_offset = [1.2]
        self.assertRaises(TypeError, list_graveyard_offset)

    def test_compound_in_shapes(self):
        shape1 = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        shape2 = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)])
        shape3 = paramak.Shape()
        shape3.solid = cq.Compound.makeCompound(
            [a.val() for a in [shape1.solid, shape2.solid]]
        )
        test_reactor = paramak.Reactor([shape3])
        assert test_reactor.solid is not None

    def test_sector_wedge_with_360_returns_none(self):
        """Trys to make a sector wedge with full 360 degree rotation and checks
        that None is returned"""

        test_shape = paramak.RotateStraightShape(
            points=[(0, 0), (0, 20), (20, 20)]
        )
        my_reactor = paramak.Reactor([test_shape])
        assert my_reactor.make_sector_wedge(rotation_angle=360) is None

    def test_reactor_volume(self):
        """Checks the types returned by the .volume method are correct"""

        assert isinstance(self.test_reactor.volume(), list)
        assert isinstance(self.test_reactor.volume(), list)
        assert isinstance(self.test_reactor.volume()[0], float)
        assert isinstance(self.test_reactor_2.volume()[0], float)
        assert isinstance(self.test_reactor_2.volume()[1], float)
        assert len(self.test_reactor.volume()) == 1
        assert len(self.test_reactor_2.volume()) == 2
        assert sum(
            self.test_reactor_2.volume()) > sum(
            self.test_reactor.volume())
        assert self.test_reactor_2.volume()[0] == self.test_reactor.volume()[0]

    def test_reactor_volume_spliting_compounds(self):
        """Checks the volumes returned by the .volume method with splitting of
        compounds set to True are correct"""

        assert isinstance(
            self.test_reactor_3.volume(
                split_compounds=True), list)
        assert isinstance(
            self.test_reactor_3.volume(
                split_compounds=False), list)
        assert isinstance(
            self.test_reactor_3.volume(
                split_compounds=True)[0], list)
        assert isinstance(
            self.test_reactor_3.volume(
                split_compounds=True)[1], list)
        assert isinstance(
            self.test_reactor_3.volume(
                split_compounds=False)[0], float)
        assert isinstance(
            self.test_reactor_3.volume(
                split_compounds=False)[1], float)
        assert isinstance(
            self.test_reactor_3.volume(
                split_compounds=True)[1][0], float)
        assert isinstance(
            self.test_reactor_3.volume(
                split_compounds=True)[1][1], float)
        assert len(self.test_reactor_3.volume(split_compounds=True)) == 2
        assert len(self.test_reactor_3.volume(split_compounds=True)[1]) == 2

        vol_1 = self.test_reactor_3.volume(split_compounds=True)[1][0]
        vol_2 = self.test_reactor_3.volume(split_compounds=True)[1][1]
        assert vol_1 == vol_2
        vol_3 = self.test_reactor_3.volume(split_compounds=False)[1]
        assert pytest.approx(vol_3, vol_1 + vol_2)

    def test_reactor_names(self):
        'checks that the names attribute returns the expected results'
        assert self.test_reactor_2.name == ['test_shape', 'test_shape2']


if __name__ == "__main__":
    unittest.main()





import unittest

import paramak
import pytest


class TestRotatedIsoscelesTriangle(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.RotatedIsoscelesTriangle(
            height=10,
            base_length=20,
            pivot_angle=0,
            pivot_point=(100, 50),
            rotation_angle=45,
        )

    def test_component_creation(self):
        """Creates a RotatedTrapezoid object and checks that the .solid is not
        None."""

        assert self.test_shape.solid is not None

    def test_check_number_of_surfaces(self):
        """Counts the surfaces in a fully rotated and partly rotated
        RotatedTrapezoid."""

        assert len(self.test_shape.areas) == 5

        self.test_shape.rotation_angle = 360

        assert len(self.test_shape.areas) == 3

    def test_args_do_not_impact_volume(self):
        """Changes args that should not impact the volume and checks that they
        do not impact the volume"""

        test_shape_vol = self.test_shape.volume()

        self.test_shape.pivot_angle = 180

        assert pytest.approx(
            self.test_shape.volume(),
            rel=0.01) == test_shape_vol

        self.test_shape.pivot_point = (100, 300)

        assert pytest.approx(
            self.test_shape.volume(),
            rel=0.01) == test_shape_vol

    def test_args_impact_volume(self):
        """Changes args that should impact the volume and checks that the
        volume changes as a result of the argument changes"""

        test_shape_vol = self.test_shape.volume()

        self.test_shape.pivot_angle = 45

        assert self.test_shape.volume() > test_shape_vol

        self.test_shape.height = 42

        assert self.test_shape.volume() > test_shape_vol

        self.test_shape.height = 10
        self.test_shape.length_2 = 42

        assert self.test_shape.volume() > test_shape_vol

        self.test_shape.height = 10
        self.test_shape.base_length = 20
        self.test_shape.length_3 = 42

        assert self.test_shape.volume() > test_shape_vol




from typing import Optional, Tuple
from paramak import RotateMixedShape


class CenterColumnShieldFlatTopHyperbola(RotateMixedShape):
    """A center column shield volume with a hyperbolic outer profile joined to
    flat profiles at the top and bottom of the shield, and a constant
    cylindrical inner profile.

    Args:
        height: height of the center column shield.
        arc_height: height of the outer hyperbolic profile of the center column
            shield.
        inner_radius: the inner radius of the center column shield.
        mid_radius: the inner radius of the outer hyperbolic profile of the
            center column shield.
        outer_radius: the outer_radius of the center column shield.
        name: Defaults to "center_column".
    """

    def __init__(
        self,
        height: float,
        arc_height: float,
        inner_radius: float,
        mid_radius: float,
        outer_radius: float,
        name: Optional[str] = "center_column",
        color: Optional[Tuple[float, float, float,
                              Optional[float]]] = (0., 0.333, 0.),
        **kwargs
    ) -> None:

        super().__init__(
            name=name,
            color=color,
            **kwargs
        )

        self.height = height
        self.arc_height = arc_height
        self.mid_radius = mid_radius
        self.outer_radius = outer_radius
        self.inner_radius = inner_radius

    @property
    def height(self):
        return self._height

    @height.setter
    def height(self, height):
        self._height = height

    @property
    def arc_height(self):
        return self._arc_height

    @arc_height.setter
    def arc_height(self, arc_height):
        self._arc_height = arc_height

    @property
    def inner_radius(self):
        return self._inner_radius

    @inner_radius.setter
    def inner_radius(self, inner_radius):
        self._inner_radius = inner_radius

    @property
    def mid_radius(self):
        return self._mid_radius

    @mid_radius.setter
    def mid_radius(self, mid_radius):
        self._mid_radius = mid_radius

    @property
    def outer_radius(self):
        return self._outer_radius

    @outer_radius.setter
    def outer_radius(self, outer_radius):
        self._outer_radius = outer_radius

    def find_points(self):
        """Finds the XZ points and connection types (straight and spline) that
        describe the 2D profile of the center column shield shape."""

        if not self.inner_radius <= self.mid_radius <= self.outer_radius:
            raise ValueError('inner_radius must be less than mid_radius. \
                mid_radius must be less than outer_radius.')

        if self.arc_height >= self.height:
            msg = ('arc_height ({self.arc_height}) is larger than height '
                   '({self.height})')
            raise ValueError(msg)

        points = [
            (self.inner_radius, 0, "straight"),
            (self.inner_radius, self.height / 2, "straight"),
            (self.outer_radius, self.height / 2, "straight"),
            (self.outer_radius, self.arc_height / 2, "spline"),
            (self.mid_radius, 0, "spline"),
            (self.outer_radius, -self.arc_height / 2, "straight"),
            (self.outer_radius, -self.height / 2, "straight"),
            (self.inner_radius, -self.height / 2, "straight")
        ]

        self.points = points




from math import pi

import pytest
from attr.setters import NO_OP
from paramak import ToroidalFieldCoilRectangleRoundCorners

obj = ToroidalFieldCoilRectangleRoundCorners(
    lower_inner_coordinates=(50, 0),
    mid_point_coordinates=(100, 100),
    thickness=20,
    distance=10,
    number_of_coils=1,
    with_inner_leg=False
)

obj2 = ToroidalFieldCoilRectangleRoundCorners(
    lower_inner_coordinates=(50, 0),
    mid_point_coordinates=(100, 100),
    thickness=20,
    distance=10,
    number_of_coils=1,
    with_inner_leg=True
)


def surface_area(
        lower_left,
        middle_right,
        thickness,
        extrusion_length,
        with_inner_leg=False,
        xz_face_only=False,
        extrusion_area_only=False):
    """
    Function calculates the total surface area of the TF coil from the
    coordinates given in the find_points function
    """
    test_object = ToroidalFieldCoilRectangleRoundCorners(
        lower_inner_coordinates=lower_left,
        mid_point_coordinates=middle_right,
        thickness=thickness,
        distance=extrusion_length,
        number_of_coils=1,
        with_inner_leg=with_inner_leg
    )

    analyse_attributes = test_object.analyse_attributes

    base, height, inner_rad, outter_rad = analyse_attributes

    # The surface area of the face in XZ plane is divisible into 5 segments
    base_segment_area = thickness * (base - inner_rad)
    vertical_segment_area = thickness * (height - (inner_rad * 2))
    corner_area = (pi / 4) * (outter_rad**2 - inner_rad**2)

    # XZ plane face area
    total_face_area = base_segment_area * 2 + \
        vertical_segment_area + corner_area * 2
    # The surface area of the planes in YZ plane
    contour_length = inner_rad * \
        (pi - 8) + pi * outter_rad + 2 * (2 * base + thickness + height)
    extrusion_area = contour_length * extrusion_length
    # Total Area
    total_bounding_surface = total_face_area * 2 + extrusion_area

    if with_inner_leg:
        # XZ plane face area
        total_face_area += thickness * height
        print(total_face_area)
        total_leg_surface_area = 2 * height * thickness + 2 * \
            extrusion_length * height + 2 * extrusion_length * thickness

        # Total bounding surface
        total_bounding_surface += total_leg_surface_area
        print("Face area: {}\nExtrusion Ares: {}\ntotal bounding area: {}"
              .format(total_face_area, extrusion_area, total_bounding_surface))

    if xz_face_only:
        return total_face_area

    if extrusion_area_only:
        return extrusion_area

    return total_bounding_surface


def volume(
        lower_left,
        middle_right,
        thickness,
        extrusion_length,
        with_inner_leg=False,
):
    """
    The function calculates the volume from the given coordinates used for
    parametarising the component in find_points function in core module it
    takes an additional variable for extrusion length which is the thickness
    of the coil
    """

    face_area = surface_area(
        lower_left,
        middle_right,
        thickness,
        extrusion_length,
        xz_face_only=True,
        with_inner_leg=with_inner_leg)
    print(face_area)
    total_shape_volume = face_area * extrusion_length

    return total_shape_volume


# Parametric Tests

@pytest.mark.parametric
def test_parametric_surface_area_with_leg():
    paramak_area = obj2.area
    package_area = surface_area(
        (50, 0), (100, 100), 20, 10, with_inner_leg=True)
    assert pytest.approx(package_area) == paramak_area


@pytest.mark.parametric
def test_parametric_volume_with_leg():
    paramak_vol = obj2.volume()
    package_vol = volume((50, 0), (100, 100), 20, 10, with_inner_leg=True)
    assert pytest.approx(package_vol) == paramak_vol


@pytest.mark.parametric
def test_parametric_surface_area():
    paramak_area = obj.area
    package_area = surface_area((50, 0), (100, 100), 20, 10)
    assert pytest.approx(package_area) == paramak_area


@pytest.mark.parametric
def test_parametric_volume():
    paramak_vol = obj.volume()
    package_vol = volume((50, 0), (100, 100), 20, 10)
    assert pytest.approx(package_vol) == paramak_vol


# Analytical Tests

@pytest.mark.analytical
def test_manual_area():
    analytical = 19872.92
    computational = surface_area((50, 0), (100, 100), 20, 10)
    assert pytest.approx(computational) == analytical


@pytest.mark.analytical
def test_manual_volume():
    analytical = 64909.73
    computational = volume((50, 0), (100, 100), 20, 10)
    assert pytest.approx(computational) == analytical


# Input Parameter Tests
# Lower Point Coordinate

@pytest.mark.dtype
def test_input_param_lower_inner():
    with pytest.raises(TypeError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=1,
            mid_point_coordinates=(100, 100),
            thickness=20,
            distance=10,
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.length
def test_input_lower_point_tuple():
    with pytest.raises(ValueError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, 0, 5),
            mid_point_coordinates=(100, 100),
            thickness=20,
            distance=10,
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.dtype
def test_input_tuple_element_type_lower_point():
    with pytest.raises(TypeError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, "string"),
            mid_point_coordinates=(100, 100),
            thickness=20,
            distance=10,
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.dtype
def test_setter_lower_point():
    with pytest.raises(TypeError):
        obj2.lower_inner_coordinates = 1


@pytest.mark.length
def test_input_lower_point_tuple_length_with_setter():
    with pytest.raises(ValueError):
        obj2.lower_inner_coordinates = (0, 0, 0)


@pytest.mark.dtype
def test_lower_point_setter_elements_z():
    with pytest.raises(TypeError):
        obj.lower_inner_coordinates = (10, "string")


@pytest.mark.dtype
def test_lower_point_setter_elements_x():
    with pytest.raises(TypeError):
        obj.lower_inner_coordinates = ("string", 10)


# Mid Point Coordinate

@pytest.mark.dtype
def test_input_param_mid_point():
    with pytest.raises(TypeError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, 0),
            mid_point_coordinates=1,
            thickness=20,
            distance=10,
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.length
def test_input_tuple2():
    with pytest.raises(ValueError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, 0),
            mid_point_coordinates=(100, 100, 100),
            thickness=20,
            distance=10,
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.dtype
def test_input_tuple_element_type_mid_point():
    with pytest.raises(TypeError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, 0),
            mid_point_coordinates=(100, "string"),
            thickness=20,
            distance=10,
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.dtype
def test_setter_mid_point():
    with pytest.raises(TypeError):
        obj.mid_point_coordinates = 1


@pytest.mark.length
def test_input_mid_point_tuple_length_with_setter():
    with pytest.raises(ValueError):
        obj2.mid_point_coordinates = (0, 0, 0)


@pytest.mark.dtype
def test_mid_point_setter_elements_z():
    with pytest.raises(TypeError):
        obj.mid_point_coordinates = (10, "string")


@pytest.mark.dtype
def test_mid_point_setter_elements_x():
    with pytest.raises(TypeError):
        obj.mid_point_coordinates = ("string", 10)


# Coordinate Comparison

@pytest.mark.value
def test_input_x_coordinates():
    with pytest.raises(ValueError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, 0),
            mid_point_coordinates=(0, 100),
            thickness=20,
            distance=10,
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.value
def test_x_coords_with_setters():
    with pytest.raises(ValueError):
        obj.lower_inner_coordinates = (150, 0)
        obj.mid_point_coordinates = (100, 0)


# Thickness Parameter

@pytest.mark.dtype
def test_input_param_thickness():
    with pytest.raises(TypeError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, 0),
            mid_point_coordinates=(100, 100),
            thickness="fail",
            distance=10,
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.dtype
def test_input_param_thickness2():
    test_object = ToroidalFieldCoilRectangleRoundCorners(
        lower_inner_coordinates=(50, 0),
        mid_point_coordinates=(100, 100),
        thickness=50,
        distance=10,
        number_of_coils=1,
    )
    inner_radius = test_object.analyse_attributes[2]
    outer_radius = test_object.analyse_attributes[3]

    check_inner = 50 * 0.1
    check_outer = 50 * 1.1
    assert inner_radius == check_inner
    assert outer_radius == check_outer


@pytest.mark.dtype
def test_thickness_setter():
    with pytest.raises(TypeError):
        obj.thickness = "string"


# Extrusion Distance

@pytest.mark.dtype
def test_input_param_distance():
    with pytest.raises(TypeError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, 0),
            mid_point_coordinates=(100, 100),
            thickness=20,
            distance="fail",
            number_of_coils=1,
        )
        assert test_object.solid is not None


@pytest.mark.dtype
def test_distance_setter():
    with pytest.raises(TypeError):
        obj.distance = "string"


# Number of Coils

@pytest.mark.value
def test_input_num_coils():
    with pytest.raises(TypeError):
        test_object = ToroidalFieldCoilRectangleRoundCorners(
            lower_inner_coordinates=(50, 0),
            mid_point_coordinates=(100, 100),
            thickness=20,
            distance=10,
            number_of_coils=1.5,
        )
        assert test_object.solid is not None


@pytest.mark.dtype
def test_num_coil_setter():
    with pytest.raises(TypeError):
        obj2.number_of_coils = 1.4




from collections.abc import Iterable
from typing import List, Optional, Tuple

from paramak import (CenterColumnShieldCircular, CenterColumnShieldCylinder,
                     CenterColumnShieldFlatTopCircular,
                     CenterColumnShieldFlatTopHyperbola,
                     CenterColumnShieldHyperbola,
                     CenterColumnShieldPlasmaHyperbola, RotateMixedShape)
import paramak


class InboardFirstwallFCCS(RotateMixedShape):
    """An inboard firstwall component that builds a constant thickness layer
    from the central column shield. The center column shields can be of type:
    CenterColumnShieldCylinder, CenterColumnShieldHyperbola,
    CenterColumnShieldFlatTopHyperbola, CenterColumnShieldCircular,
    CenterColumnShieldPlasmaHyperbola or CenterColumnShieldFlatTopCircular

    Args:
        central_column_shield: The central column shield object to build from
        thickness: the radial thickness of the firstwall (cm)
    """

    def __init__(
        self,
        central_column_shield: paramak.Shape,
        thickness: float,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (0.5, 0.5, 0.5),
        **kwargs
    ):

        super().__init__(
            color=color,
            **kwargs
        )

        self.central_column_shield = central_column_shield
        self.thickness = thickness

    @property
    def thickness(self):
        return self._thickness

    @thickness.setter
    def thickness(self, value):
        self._thickness = value

    @property
    def central_column_shield(self):
        return self._central_column_shield

    @central_column_shield.setter
    def central_column_shield(self, value):
        self._central_column_shield = value

    def find_points(self):

        # check that is an acceptable class
        acceptable_classes = (
            CenterColumnShieldCylinder,
            CenterColumnShieldHyperbola,
            CenterColumnShieldFlatTopHyperbola,
            CenterColumnShieldPlasmaHyperbola,
            CenterColumnShieldCircular,
            CenterColumnShieldFlatTopCircular
        )
        if not isinstance(self.central_column_shield, acceptable_classes):
            raise ValueError(
                "InboardFirstwallFCCS.central_column_shield must be an \
                instance of CenterColumnShieldCylinder, \
                CenterColumnShieldHyperbola, \
                CenterColumnShieldFlatTopHyperbola, \
                CenterColumnShieldPlasmaHyperbola, \
                CenterColumnShieldCircular, CenterColumnShieldFlatTopCircular")

        inner_radius = self.central_column_shield.inner_radius
        height = self.central_column_shield.height

        if isinstance(self.central_column_shield, CenterColumnShieldCylinder):
            firstwall = CenterColumnShieldCylinder(
                height=height,
                inner_radius=inner_radius,
                outer_radius=self.central_column_shield.outer_radius +
                self.thickness
            )

        elif isinstance(self.central_column_shield,
                        CenterColumnShieldHyperbola):
            firstwall = CenterColumnShieldHyperbola(
                height=height,
                inner_radius=inner_radius,
                mid_radius=self.central_column_shield.mid_radius +
                self.thickness,
                outer_radius=self.central_column_shield.outer_radius +
                self.thickness,
            )

        elif isinstance(self.central_column_shield,
                        CenterColumnShieldFlatTopHyperbola):
            firstwall = CenterColumnShieldFlatTopHyperbola(
                height=height,
                arc_height=self.central_column_shield.arc_height,
                inner_radius=inner_radius,
                mid_radius=self.central_column_shield.mid_radius +
                self.thickness,
                outer_radius=self.central_column_shield.outer_radius +
                self.thickness,
            )

        elif isinstance(self.central_column_shield,
                        CenterColumnShieldPlasmaHyperbola):
            firstwall = CenterColumnShieldPlasmaHyperbola(
                height=height,
                inner_radius=inner_radius,
                mid_offset=self.central_column_shield.mid_offset -
                self.thickness,
                edge_offset=self.central_column_shield.edge_offset -
                self.thickness,
            )

        elif isinstance(self.central_column_shield,
                        CenterColumnShieldCircular):
            firstwall = CenterColumnShieldCircular(
                height=height,
                inner_radius=inner_radius,
                mid_radius=self.central_column_shield.mid_radius +
                self.thickness,
                outer_radius=self.central_column_shield.outer_radius +
                self.thickness,
            )

        elif isinstance(self.central_column_shield,
                        CenterColumnShieldFlatTopCircular):
            firstwall = CenterColumnShieldFlatTopCircular(
                height=height,
                arc_height=self.central_column_shield.arc_height,
                inner_radius=inner_radius,
                mid_radius=self.central_column_shield.mid_radius +
                self.thickness,
                outer_radius=self.central_column_shield.outer_radius +
                self.thickness,
            )

        firstwall.rotation_angle = self.rotation_angle
        self.connection_type = firstwall.connection_type
        self.points = firstwall.points

        # add to cut attribute
        if self.cut is None:
            self.cut = self.central_column_shield
        elif isinstance(self.cut, Iterable):
            self.cut = [*self.cut, self.central_column_shield]
        else:
            self.cut = [*[self.cut], self.central_column_shield]




import unittest

import numpy as np
import paramak


class TestPortCutterRotated(unittest.TestCase):
    def test_shape_construction_and_volume(self):
        """Cuts a vessel cylinder with several different size port cutters."""

        small_ports = paramak.PortCutterRotated(
            polar_coverage_angle=5,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=1000,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            rotation_angle=10
        )

        large_ports = paramak.PortCutterRotated(
            polar_coverage_angle=6,
            center_point=(100, 0),
            polar_placement_angle=10,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            max_distance_from_center=1000,
            rotation_angle=10
        )

        vessel_with_out_ports = paramak.CenterColumnShieldCylinder(
            height=500,
            inner_radius=200,
            outer_radius=300
        )

        vessel_with_small_ports = paramak.CenterColumnShieldCylinder(
            height=500,
            inner_radius=200,
            outer_radius=300,
            cut=small_ports
        )

        vessel_with_large_ports = paramak.CenterColumnShieldCylinder(
            height=500,
            inner_radius=200,
            outer_radius=300,
            cut=large_ports
        )

        assert large_ports.volume() > small_ports.volume()
        assert vessel_with_out_ports.volume() > vessel_with_small_ports.volume()
        assert vessel_with_small_ports.volume() > vessel_with_large_ports.volume()

    def test_polar_coverage_angle_impacts_volume(self):
        """Checks the volumes of two port cutters with different
        polar_coverage_angle and checks angle impacts the volume."""

        small_ports = paramak.PortCutterRotated(
            polar_coverage_angle=5,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=1000,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            rotation_angle=10
        )

        large_ports = paramak.PortCutterRotated(
            polar_coverage_angle=6,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=1000,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            rotation_angle=10
        )

        assert large_ports.volume() > small_ports.volume()

    def test_max_distance_from_center_impacts_volume(self):
        """Checks the volumes of two port cutters with different
        max_distance_from_center and checks distance impacts the volume."""

        small_ports = paramak.PortCutterRotated(
            polar_coverage_angle=5,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=1000,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            rotation_angle=10
        )

        large_ports = paramak.PortCutterRotated(
            polar_coverage_angle=5,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=2000,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            rotation_angle=10
        )

        assert large_ports.volume() > small_ports.volume()

    def test_azimuth_placement_angle_impacts_volume(self):
        """Checks the volumes of two port cutters with different
        azimuth_placement_angle and checks distance impacts the volume."""

        small_ports = paramak.PortCutterRotated(
            polar_coverage_angle=5,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=1000,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            rotation_angle=10
        )

        large_ports = paramak.PortCutterRotated(
            polar_coverage_angle=5,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=1000,
            azimuth_placement_angle=np.linspace(0, 360, 5),
            rotation_angle=10
        )

        assert large_ports.volume() > small_ports.volume()

    def test_rotation_angle_impacts_volume(self):
        """Checks the volumes of two port cutters with different
        rotation_angle and checks distance impacts the volume."""

        small_ports = paramak.PortCutterRotated(
            polar_coverage_angle=5,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=1000,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            rotation_angle=10
        )

        large_ports = paramak.PortCutterRotated(
            polar_coverage_angle=5,
            center_point=(100, 0),
            polar_placement_angle=10,
            max_distance_from_center=1000,
            azimuth_placement_angle=np.linspace(0, 360, 4),
            rotation_angle=20
        )

        assert large_ports.volume() > small_ports.volume()

    def test_outerpoint_negative(self):
        """Tests that when polar_coverage_angle is greater than 180 an error is
        raised."""
        def error():
            paramak.PortCutterRotated(
                center_point=(1, 1),
                polar_coverage_angle=181,
                polar_placement_angle=0,
                rotation_angle=10,
            )
        self.assertRaises(ValueError, error)




import unittest

import numpy as np
import paramak


class TestCuttingWedgeFS(unittest.TestCase):

    def test_shape_construction_and_volume(self):
        """Makes cutting cylinders from shapes and checks the
        volume of the cutter shape is larger than the shape it
        encompasses."""

        hoop_shape = paramak.PoloidalFieldCoil(height=20,
                                               width=20,
                                               center_point=(50, 200),
                                               rotation_angle=180)

        cutter = paramak.CuttingWedgeFS(
            shape=hoop_shape,
            azimuth_placement_angle=0,
        )

        assert cutter.volume() > hoop_shape.volume()

    def test_invalid_parameters_errors(self):
        """Checks that the correct errors are raised when invalid arguments are input as
        shape parameters."""

        shape = paramak.ExtrudeStraightShape(
            distance=1,
            points=[(0, 0), (0, 1), (1, 1)],
            rotation_angle=180
        )

        cutter = paramak.CuttingWedgeFS(
            shape=shape,
            azimuth_placement_angle=0,
        )

        def incorrect_rotation_angle():
            shape.rotation_angle = 360
            cutter.solid

        def incorrect_shape_points():
            shape.rotation_angle = 180
            cutter.shape.points = [(0, 0, 'straight')]
            cutter.solid

        def incorrect_shape_rotation_angle():
            cutter.shape.points = [(0, 0), (0, 1), (1, 1)]
            shape.rotation_angle = 360
            cutter.shape = shape

        self.assertRaises(ValueError, incorrect_rotation_angle)
        self.assertRaises(ValueError, incorrect_shape_points)
        self.assertRaises(ValueError, incorrect_shape_rotation_angle)

    def test_different_workplanes(self):
        """Test that checks the cutting wedge can be correctly applied to a
        shape with non-default workplane and rotation_axis
        """
        rectangle = paramak.ExtrudeStraightShape(
            2,
            points=[(-0.5, -0.5), (0.5, -0.5), (0.5, 0.5), (-0.5, 0.5)],
            workplane="XY",
            rotation_axis="Z"
        )
        rectangle.rotation_angle = 360
        volume_full = rectangle.volume()
        assert np.isclose(volume_full, 2)
        rectangle.rotation_angle = 90
        volume_quarter = rectangle.volume()
        assert np.isclose(volume_quarter, 0.5)




import unittest

import paramak


class TestCenterColumnShieldHyperbola(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.CenterColumnShieldHyperbola(
            height=600, inner_radius=100, mid_radius=150, outer_radius=200
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a CenterColumnShieldHyperbola are
        correct."""

        assert self.test_shape.rotation_angle == 360

    def test_processed_points_calculation(self):
        """Checks that the processed_points used to construct the
        CenterColumnShieldHyperbola component are calculated correctly from the
        parameters given."""

        assert self.test_shape.processed_points == [
            (100.0, 0.0, "straight"),
            (100.0, 300.0, "straight"),
            (200.0, 300.0, "spline"),
            (150.0, 0.0, "spline"),
            (200.0, -300.0, "straight"),
            (100.0, -300.0, "straight"),
            (100.0, 0.0, "straight")
        ]

    def test_points_calculation(self):
        """Checks that the points used to construct the
        CenterColumnShieldHyperbola component are calculated correctly from the
        parameters given."""

        assert self.test_shape.points == [
            (100.0, 0.0, "straight"),
            (100.0, 300.0, "straight"),
            (200.0, 300.0, "spline"),
            (150.0, 0.0, "spline"),
            (200.0, -300.0, "straight"),
            (100.0, -300.0, "straight")
        ]

    def test_creation(self):
        """Creates a center column shield using the
        CenterColumnShieldHyperbola parametric component and checks that a
        cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_invalid_parameters_errors(self):
        """Checks that the correct errors are raised when invalid arguments are input
        as shape parameters."""

        def incorrect_inner_radius():
            self.test_shape.inner_radius = 180
            self.test_shape.solid

        def incorrect_mid_radius():
            self.test_shape.inner_radius = 100
            self.test_shape.mid_radius = 80
            self.test_shape.solid

        def incorrect_outer_radius():
            self.test_shape.mid_radius = 150
            self.test_shape.outer_radius = 130
            self.test_shape.solid

        self.assertRaises(ValueError, incorrect_inner_radius)
        self.assertRaises(ValueError, incorrect_mid_radius)
        self.assertRaises(ValueError, incorrect_outer_radius)

    def test_faces(self):
        """Creates a center column shield using the CenterColumnShieldHyperbola
        parametric component and checks that a solid with the correct number of
        faces is created"""

        assert len(self.test_shape.areas) == 4
        assert len(set([round(i) for i in self.test_shape.areas])) == 3

        self.test_shape.rotation_angle = 180
        assert len(self.test_shape.areas) == 6
        assert len(set([round(i) for i in self.test_shape.areas])) == 4




import unittest

import paramak


class TestCenterColumnShieldCircular(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.CenterColumnShieldCircular(
            height=600, inner_radius=100, mid_radius=150, outer_radius=200
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a CenterColumnShieldCircular are correct."""

        assert self.test_shape.rotation_angle == 360

    def test_points_calculation(self):
        """Checks that the points used to construct the CenterColumnShieldCircular components
        are calculated correctly from the parameters given."""

        assert self.test_shape.points == [
            (100, 0, 'straight'),
            (100, 300.0, 'straight'),
            (200, 300.0, 'circle'),
            (150, 0, 'circle'),
            (200, -300.0, 'straight'),
            (100, -300.0, 'straight'),
        ]

    def test_processed_points_calculation(self):
        """Checks that the processed_points used to construct the CenterColumnShieldCircular components
        are calculated correctly from the parameters given."""

        assert self.test_shape.processed_points == [
            (100, 0, 'straight'),
            (100, 300.0, 'straight'),
            (200, 300.0, 'circle'),
            (150, 0, 'circle'),
            (200, -300.0, 'straight'),
            (100, -300.0, 'straight'),
            (100, 0, 'straight')
        ]

    def test_creation(self):
        """Creates a center column shield using the CenterColumnShieldCircular
        parametric component and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_faces(self):
        """Creates a center column shield using the CenterColumnShieldCircular
        parametric component and checks that a solid is created with the correct
        number of faces"""

        assert len(self.test_shape.areas) == 4
        assert len(set([round(i) for i in self.test_shape.areas])) == 3

        self.test_shape.rotation_angle = 180
        assert len(self.test_shape.areas) == 6
        assert len(set([round(i) for i in self.test_shape.areas])) == 4




import math
import unittest

import paramak
import pytest


class TestPoloidalFieldCoilSet(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.PoloidalFieldCoilSet(
            heights=[10, 15, 5],
            widths=[20, 25, 30],
            center_points=[(100, 100), (200, 200), (300, 300)]
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a PoloidalFieldCoilSet are correct."""

        assert self.test_shape.rotation_angle == 360

    def test_points_calculation(self):
        """Checks that the points used to construct the PoloidalFieldCoilSet
        are calculated correctly from the parameters given."""

        assert self.test_shape.points == [
            (110.0, 105.0),
            (110.0, 95.0),
            (90.0, 95.0),
            (90.0, 105.0),
            (212.5, 207.5),
            (212.5, 192.5),
            (187.5, 192.5),
            (187.5, 207.5),
            (315.0, 302.5),
            (315.0, 297.5),
            (285.0, 297.5),
            (285.0, 302.5),
        ]

    def test_processed_points_calculation(self):
        """Checks that the processed_points used to construct the
        PoloidalFieldCoilSet are calculated correctly from the parameters
        given."""

        assert self.test_shape.processed_points == [
            (110.0, 105.0, 'straight'),
            (110.0, 95.0, 'straight'),
            (90.0, 95.0, 'straight'),
            (90.0, 105.0, 'straight'),
            (212.5, 207.5, 'straight'),
            (212.5, 192.5, 'straight'),
            (187.5, 192.5, 'straight'),
            (187.5, 207.5, 'straight'),
            (315.0, 302.5, 'straight'),
            (315.0, 297.5, 'straight'),
            (285.0, 297.5, 'straight'),
            (285.0, 302.5, 'straight'),
            (110.0, 105.0, 'straight')
        ]

    def test_creation(self):
        """Creates a solid using the PoloidalFieldCoilSet parametric component
        and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert len(self.test_shape.solid.Solids()) == 3

    def test_absolute_volume(self):
        """Creates a set of pf coils using the PoloidalFieldCoilSet parametric
        component and checks that the volume is correct."""

        assert self.test_shape.volume() == (pytest.approx((10 * 20 * math.pi * (2 * 100)) + (
            15 * 25 * math.pi * (2 * 200)) + (5 * 30 * math.pi * (2 * 300))))

    def test_absolute_areas(self):
        """Creates a set of pf coils using the PoloidalFieldCoilSet parametric
        component and checks that the areas of its faces are correct."""

        assert len(self.test_shape.areas) == 12
        assert len(set(round(i) for i in self.test_shape.areas)) == 9
        assert self.test_shape.areas.count(
            pytest.approx(20 * math.pi * (2 * 100))) == 2
        assert self.test_shape.areas.count(
            pytest.approx(25 * math.pi * (2 * 200))) == 2
        assert self.test_shape.areas.count(
            pytest.approx(30 * math.pi * (2 * 300))) == 2
        assert self.test_shape.areas.count(
            pytest.approx(10 * math.pi * (2 * 90))) == 1
        assert self.test_shape.areas.count(
            pytest.approx(10 * math.pi * (2 * 110))) == 1
        assert self.test_shape.areas.count(
            pytest.approx(15 * math.pi * (2 * 187.5))) == 1
        assert self.test_shape.areas.count(
            pytest.approx(15 * math.pi * (2 * 212.5))) == 1
        assert self.test_shape.areas.count(
            pytest.approx(5 * math.pi * (2 * 285))) == 1
        assert self.test_shape.areas.count(
            pytest.approx(5 * math.pi * (2 * 315))) == 1

    def test_PoloidalFieldCoilSet_incorrect_height(self):
        """Checks that an error is raised when a PoloidalFieldCoilSet is made
        with height passed as the wrong type."""

        def make_PoloidalFieldCoilSet_incorrect_height():
            paramak.PoloidalFieldCoilSet(
                heights=10,
                widths=[20, 20, 20],
                center_points=[(100, 100), (200, 200), (300, 300)])

        self.assertRaises(
            ValueError,
            make_PoloidalFieldCoilSet_incorrect_height
        )

    def test_PoloidalFieldCoilSet_incorrect_width(self):
        """Checks that an error is raised when a PoloidalFieldCoilSet is made
        with width passed as the wrong type."""

        def make_PoloidalFieldCoilSet_incorrect_width():
            paramak.PoloidalFieldCoilSet(
                heights=[10, 10, 10],
                widths=20,
                center_points=[(100, 100), (200, 200), (300, 300)])

        self.assertRaises(
            ValueError,
            make_PoloidalFieldCoilSet_incorrect_width
        )

    def test_PoloidalFieldCoilSet_incorrect_center_points(self):
        """Checks that an error is raised when a PoloidalFieldCoilSet is made
        with center_points passed as the wrong type."""

        def make_PoloidalFieldCoilSet_incorrect_center_points():
            paramak.PoloidalFieldCoilSet(
                heights=[10, 10, 10],
                widths=[20, 20, 20],
                center_points=100)

        self.assertRaises(
            ValueError,
            make_PoloidalFieldCoilSet_incorrect_center_points
        )

    def test_PoloidalFieldCoilSet_incorrect_width_length(self):
        """Checks that an error is raised when a PoloidalFieldCoilSet is made
        with the incorrect number of widths."""

        def make_PoloidalFieldCoilSet_incorrect_width_length():
            paramak.PoloidalFieldCoilSet(
                heights=[10, 10, 10],
                widths=[20, 20],
                center_points=[(100, 100), (200, 200), (300, 300)])

        self.assertRaises(
            ValueError,
            make_PoloidalFieldCoilSet_incorrect_width_length
        )




from typing import Optional, Tuple

from paramak import Shape
from paramak.utils import calculate_wedge_cut


class ExtrudeMixedShape(Shape):
    """Extrudes a 3d CadQuery solid from points connected with a mixture of
    straight and spline connections.

    Args:
        distance: the extrusion distance to use (cm units if used for
            neutronics)
        extrude_both: If set to True, the extrusion will occur in both
            directions. Defaults to True.
        rotation_angle: rotation angle of solid created. A cut is performed
            from rotation_angle to 360 degrees. Defaults to 360.0.
    """

    def __init__(
        self,
        distance: float,
        extrude_both: Optional[bool] = True,
        rotation_angle: Optional[float] = 360.0,
        extrusion_start_offset: Optional[float] = 0.0,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (0.2, 0.627, 0.172),
        **kwargs
    ):

        super().__init__(
            color=color,
            **kwargs
        )
        self.distance = distance
        self.extrude_both = extrude_both
        self.rotation_angle = rotation_angle
        self.extrusion_start_offset = extrusion_start_offset

    @property
    def distance(self):
        return self._distance

    @distance.setter
    def distance(self, value):
        self._distance = value

    @property
    def rotation_angle(self):
        return self._rotation_angle

    @rotation_angle.setter
    def rotation_angle(self, value):
        self._rotation_angle = value

    @property
    def extrusion_start_offset(self):
        return self._extrusion_start_offset

    @extrusion_start_offset.setter
    def extrusion_start_offset(self, value):
        self._extrusion_start_offset = value

    def create_solid(self):
        """Creates an extruded 3d solid using points connected with straight
        and spline edges.

           Returns:
              A CadQuery solid: A 3D solid volume
        """

        solid = super().create_solid()

        if not self.extrude_both:
            extrusion_distance = -self.distance
        else:
            extrusion_distance = -self.distance / 2.0

        wire = solid.close()

        self.wire = wire

        solid = wire.extrude(
            distance=extrusion_distance,
            both=self.extrude_both)

        # filleting rectangular port cutter edges
        # must be done before azimuthal placement
        if hasattr(self, "add_fillet"):
            solid = self.add_fillet(solid)

        solid = self.rotate_solid(solid)
        cutting_wedge = calculate_wedge_cut(self)
        solid = self.perform_boolean_operations(solid, wedge_cut=cutting_wedge)
        self.solid = solid

        return solid




import math
import unittest

import paramak
import pytest


class TestCoolantChannelRingStraight(unittest.TestCase):

    def setUp(self):
        self.test_shape = paramak.CoolantChannelRingStraight(
            height=100,
            channel_radius=10,
            ring_radius=70,
            number_of_coolant_channels=8
        )

    def test_default_parameters(self):
        """Checks that the default parameters of a CoolantChannelRingStraight are correct."""

        # assert self.test_shape.rotation_angle == 360
        assert self.test_shape.start_angle == 0

    def test_creation(self):
        """Creates a coolant channel ring using the CoolantChannelRingStraight parameteric shape
        and checks that a cadquery solid is created."""

        assert self.test_shape.solid is not None
        assert self.test_shape.volume() > 1000

    def test_faces(self):
        """Creates a CoolantChannelRingStraight shape and checks that the areas of its faces
        are correct."""

        self.test_shape.workplane = "XY"
        self.test_shape.rotation_axis = "Z"

        assert self.test_shape.area == pytest.approx(
            (((math.pi * (10**2)) * 2) + (math.pi * (10 * 2) * 100)) * 8)
        assert len(self.test_shape.areas) == 24
        assert self.test_shape.areas.count(
            pytest.approx(math.pi * (10**2))) == 16
        assert self.test_shape.areas.count(
            pytest.approx(math.pi * (10 * 2) * 100)) == 8

    def test_volume(self):
        """Creates CoolantChannelRingStraight shapes and checks that the volumes are correct."""

        self.test_shape.workplane = "XY"
        self.test_shape.rotation_axis = "Z"

        assert self.test_shape.volume() == pytest.approx(
            math.pi * (10 ** 2) * 100 * 8)

    def test_start_angle(self):
        """Checks that the coolant channels are placed at the correct azimuthal placement
        angles for a given start angle."""

        assert self.test_shape.azimuth_placement_angle == [
            0, 45, 90, 135, 180, 225, 270, 315
        ]
        self.test_shape.start_angle = 10
        assert self.test_shape.azimuth_placement_angle == [
            10, 55, 100, 145, 190, 235, 280, 325
        ]




from typing import List, Optional, Tuple

from paramak import SweepMixedShape


class SweepSplineShape(SweepMixedShape):
    """Sweeps a 2D shape created from points connected with spline connections
    along a defined spline path to create a 3D CadQuery solid. Note, some
    variation in the cross-section of the solid may occur.

    Args:
        path_points: A list of XY, YZ or XZ coordinates connected by spline
            connections which define the path along which the 2D shape is
            swept.
        workplane: Workplane in which the 2D shape to be swept is defined.
            Defaults to "XY".
        path_workplane: Workplane in which the spline path is defined. Defaults
            to "XZ".
        force_cross_section (bool, optional): If True, cross-setion of solid
            is forced to be shape defined by points in workplane at each
            path_point. Defaults to False.
    """

    def __init__(
        self,
        path_points: List[Tuple[float, float]],
        workplane: Optional[str] = "XY",
        path_workplane: Optional[str] = "XZ",
        force_cross_section: Optional[bool] = False,
        color: Optional[Tuple[float, float, float, Optional[float]]] = (0.992, 0.749, 0.435),
        **kwargs
    ):

        super().__init__(
            path_points=path_points,
            workplane=workplane,
            path_workplane=path_workplane,
            connection_type="spline",
            force_cross_section=force_cross_section,
            color=color,
            **kwargs
        )




import os
import unittest
from pathlib import Path

import paramak


class TestSparc2020Reactor(unittest.TestCase):
    """"Tests functionality of the Sparc2020Reactor class"""

    def setUp(self):
        self.test_reactor = paramak.BallReactor(
            inner_bore_radial_thickness=50,
            inboard_tf_leg_radial_thickness=200,
            center_column_shield_radial_thickness=50,
            divertor_radial_thickness=100,
            inner_plasma_gap_radial_thickness=150,
            plasma_radial_thickness=100,
            outer_plasma_gap_radial_thickness=50,
            plasma_gap_vertical_thickness=50,
            firstwall_radial_thickness=50,
            blanket_radial_thickness=100,
            blanket_rear_wall_radial_thickness=10,
            elongation=2,
            triangularity=0.55,
            number_of_tf_coils=16,
            rotation_angle=180,
        )

        self.output_filenames = [
            "plasma.stp",
            "inboard_pf_coils.stp",
            "outboard_pf_coils.stp",
            "div_coils.stp",
            "vs_coils.stp",
            "efccu_coils_1.stp",
            "efccu_coils_2.stp",
            "efccu_coils_3.stp",
            "efccu_coils_4.stp",
            "efccu_coils_5.stp",
            "efccu_coils_6.stp",
            "antenna.stp",
            "tf_coil.stp",
            "vacuum_vessel.stp",
            "vacuum_vessel_inner.stp",
        ]

    def test_input_variables_names(self):
        """tests that the number of inputs variables is correct"""

        assert len(self.test_reactor.input_variables.keys()) == 28
        assert len(self.test_reactor.input_variable_names) == 28

    def test_make_sparc_2020_reactor(self):
        """Runs the example to check the output files are produced"""
        os.system("rm *.stp")
        my_reactor = paramak.SparcFrom2020PaperDiagram()
        my_reactor.export_stp(filename=self.output_filenames)
        for output_filename in self.output_filenames:
            assert Path(output_filename).exists() is True

    def test_make_parametric_sparc_2020_rector(self):
        """Runs the example to check the output files are produced for sector
        model"""

        os.system("rm *.stp")
        my_reactor = paramak.SparcFrom2020PaperDiagram(rotation_angle=90)
        my_reactor.export_stp(filename=self.output_filenames)
        for output_filename in self.output_filenames:
            assert Path(output_filename).exists() is True




from typing import List, Optional, Union

import paramak


class CylinderReactor(paramak.Reactor):
    """Creates the 3D geometry for the cylinder reactor model based on
    parameters.

    Args:
        inner_blanket_radius: The radial distance between the center of the
            reactor on the start of the blanket (cm).
        blanket_thickness: The radial thickness of the blanket (cm).
        blanket_height: The height (z axis direction) of the blanket (cm).
        lower_blanket_thickness: The thickness (z axis direction) of the
            lower blanket pool (cm).
        blanket_vv_gap: The radial distance between the outer edge of the
            blanket and the inner edge of the vaccum vessel (cm).
        upper_vv_thickness: The thickness (z axis direction) of the
            the upper section of vaccum vessel (cm).
        vv_thickness: The radial thickness of the vaccum vessel (cm)
        lower_vv_thickness: The thickness (z axis direction) of the
            the lower section of vaccum vessel (cm).
        rotation_angle: The angle of the sector simulated. Set to 360 for
            simulations and less when creating models for visualization.
    Returns:
        paramak.Reactor object
    """

    def __init__(
        self,
        inner_blanket_radius: Optional[float] = 100,
        blanket_thickness: Optional[float] = 60,
        blanket_height: Optional[float] = 500,
        lower_blanket_thickness: Optional[float] = 50,
        upper_blanket_thickness: Optional[float] = 40,
        blanket_vv_gap: Optional[float] = 20,
        upper_vv_thickness: Optional[float] = 10,
        vv_thickness: Optional[float] = 10,
        lower_vv_thickness: Optional[float] = 10,
        rotation_angle: Optional[float] = 360,
    ):

        super().__init__([])

        self.rotation_angle = rotation_angle
        self.inner_blanket_radius = inner_blanket_radius
        self.blanket_thickness = blanket_thickness
        self.blanket_height = blanket_height
        self.lower_blanket_thickness = lower_blanket_thickness
        self.upper_blanket_thickness = upper_blanket_thickness
        self.blanket_vv_gap = blanket_vv_gap
        self.upper_vv_thickness = upper_vv_thickness
        self.vv_thickness = vv_thickness
        self.lower_vv_thickness = lower_vv_thickness

        # adds self.input_variable_names from the Reactor class
        self.input_variable_names = self.input_variable_names + [
            'inner_blanket_radius',
            'blanket_thickness',
            'blanket_height',
            'lower_blanket_thickness',
            'upper_blanket_thickness',
            'blanket_vv_gap',
            'upper_vv_thickness',
            'vv_thickness',
            'lower_vv_thickness',
            'rotation_angle',
        ]

    def create_solids(self):
        """Creates a list of paramak.Shape for components and saves it in
        self.shapes_and_components
        """
        inner_wall = (
            self.inner_blanket_radius +
            self.blanket_thickness +
            self.blanket_vv_gap)
        lower_vv = paramak.RotateStraightShape(
            points=[
                (inner_wall,
                 (-self.blanket_height / 2.0) - self.lower_blanket_thickness,
                 ),
                (inner_wall,
                 (-self.blanket_height / 2.0) - (
                     self.lower_blanket_thickness + self.lower_vv_thickness),
                 ),
                (0,
                 (-self.blanket_height / 2.0) - (
                     self.lower_blanket_thickness + self.lower_vv_thickness),
                 ),
                (0,
                 (-self.blanket_height / 2.0) - self.lower_blanket_thickness),
            ],
            rotation_angle=self.rotation_angle,
            color=(
                0.5,
                0.5,
                0.5),
            name="lower vacuum vessel",
        )

        lower_blanket = paramak.RotateStraightShape(
            points=[
                (inner_wall, -self.blanket_height / 2.0),
                (
                    inner_wall,
                    (-self.blanket_height / 2.0) - self.lower_blanket_thickness,
                ),
                (0, (-self.blanket_height / 2.0) - self.lower_blanket_thickness),
                (0, -self.blanket_height / 2.0),
            ],
            rotation_angle=self.rotation_angle,
            color=(0.0, 1.0, 0.498),
            name="lower blanket",
        )

        blanket = paramak.CenterColumnShieldCylinder(
            height=self.blanket_height,
            inner_radius=self.inner_blanket_radius,
            outer_radius=self.blanket_thickness + self.inner_blanket_radius,
            rotation_angle=self.rotation_angle,
            cut=lower_blanket,
            color=(0.0, 1.0, 0.498),
            name="blanket",
        )

        upper_blanket = paramak.RotateStraightShape(
            points=[
                (inner_wall, (self.blanket_height / 2.0) + self.upper_vv_thickness),
                (
                    inner_wall,
                    (self.blanket_height / 2.0)
                    + self.upper_vv_thickness
                    + self.upper_blanket_thickness,
                ),
                (
                    0,
                    (self.blanket_height / 2.0)
                    + self.upper_vv_thickness
                    + self.upper_blanket_thickness,
                ),
                (0, (self.blanket_height / 2.0) + self.upper_vv_thickness),
            ],
            rotation_angle=self.rotation_angle,
            color=(0.0, 1.0, 0.498),
            name="upper blanket",
        )

        upper_vv = paramak.RotateStraightShape(
            points=[
                (inner_wall, self.blanket_height / 2.0),
                (inner_wall, (self.blanket_height / 2.0) + self.upper_vv_thickness),
                (0, (self.blanket_height / 2.0) + self.upper_vv_thickness),
                (0, self.blanket_height / 2.0),
            ],
            rotation_angle=self.rotation_angle,
            color=(0.5, 0.5, 0.5),
            name="upper vacuum vessel",
        )

        vac_ves = paramak.RotateStraightShape(
            points=[
                (
                    inner_wall + self.vv_thickness,
                    (self.blanket_height / 2.0)
                    + self.upper_vv_thickness
                    + self.upper_blanket_thickness,
                ),
                (
                    inner_wall,
                    (self.blanket_height / 2.0)
                    + self.upper_vv_thickness
                    + self.upper_blanket_thickness,
                ),
                (
                    inner_wall,
                    -(self.blanket_height / 2.0)
                    - self.lower_blanket_thickness
                    - self.lower_vv_thickness,
                ),
                (
                    inner_wall + self.vv_thickness,
                    -(self.blanket_height / 2.0)
                    - self.lower_blanket_thickness
                    - self.lower_vv_thickness,
                ),
            ],
            rotation_angle=self.rotation_angle,
            color=(0.5, 0.5, 0.5),
            name="vacuum vessel",
        )

        self.shapes_and_components = [
            blanket,
            vac_ves,
            upper_blanket,
            lower_blanket,
            lower_vv,
            upper_vv,
        ]




import os
import sys
from pathlib import Path

from notebook_testing import notebook_run

sys.path.append(os.path.join(os.path.dirname(__file__), '..', 'examples'))


def main():
    for notebook in Path().rglob("examples/example_parametric_shapes/*.ipynb"):
        print(notebook)
        errors = notebook_run(notebook)
        assert errors == []


if __name__ == "__main__":
    main()
